
package cat.cordecor.crm.activity;

import java.util.ArrayList;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.drawable.BitmapDrawable;
import android.media.ThumbnailUtils;
import android.os.Bundle;
import android.os.Parcelable;
import android.text.Html;
import android.util.SparseArray;
import android.view.View;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;
import cat.cordecor.crm.R;
import cat.cordecor.crm.database.DataManager;
import cat.cordecor.crm.database.EntityComanda;
import cat.cordecor.crm.database.EntityContacte;
import cat.cordecor.crm.database.EntityEmpresa;
import cat.cordecor.crm.database.EntityProducte;
import cat.cordecor.crm.database.EntityUsuari;
import cat.cordecor.crm.http.HttpManager;
import cat.cordecor.crm.util.CurrencyLocale;
import cat.cordecor.crm.util.Log;
import cat.cordecor.crm.util.Utils;

public class ComandaEdicioActivity extends Activity {
	public static final String EXTRA_COMANDA = "comanda";
	public static final String EXTRA_EMPRESA = "empresa";
	
	private static final int SELECT_PRODUCTES = 78;
	
	private ImageView image;
	private TextView textEmpresa;
	private TextView textData;
	private TextView textDescripcio;
	private TextView textTotal;
	private ImageButton btnProductes;
	private LinearLayout btnProductesLayout;
	private ImageButton btnOk;
	private LinearLayout layoutFooter;
	private ProgressDialog progressDialog = null;
	
	private EntityUsuari user;
	private EntityEmpresa empresa;
	private EntityComanda comanda;
	private SparseArray<EntityProducte> productes;
	private String descripcio;
	private double total;
	
	private boolean modeHistory = false; 
	
	private ArrayList<ProducteLlistatActivity.ProducteQuantitat> quantitats;
	
	@Override
	public void onBackPressed() {
		super.onBackPressed();
		overridePendingTransition(R.anim.translate_right_in, R.anim.translate_right_out);
		finish();
	}
	
    @Override
    public void onCreate(Bundle savedInstanceState) {
    	super.onCreate(savedInstanceState);
    	Log.i("ContacteEdicioActivity.onCreate()");
    	
    	overridePendingTransition(R.anim.translate_left_in, R.anim.translate_left_out);
    	setContentView(R.layout.activity_comanda_edicio);
    	
    	Utils.configTopActivity(this, getString(R.string.comanda_title));
    	
    	// Recuperem les dades que ens va passar la activity del listview
    	Intent intent = getIntent();
    	empresa = intent.getParcelableExtra(EXTRA_EMPRESA);
    	comanda = intent.getParcelableExtra(EXTRA_COMANDA);
    	
    	if(comanda != null){ // Histric
    		modeHistory = true;
    		configView();
    	}
    	else{ // Nova comanda
    		modeHistory = false;
    		
        	// Recuperem les dades de l'usuari
        	Utils.executeAsync(new Utils.AsyncListener() {
    			@Override
    			public boolean runBackground() {
    				user = DataManager.getInstance().getUsuari();
    				productes = DataManager.getInstance().getProductes();
    				return (user!=null) ? true : false;
    			}
    			
    			@Override
    			public void runPost(boolean ok) {
    				configView();
    			}
    		});
    	}
    }
    
    private void configView(){
    	image = (ImageView) findViewById(R.id.image);
    	textEmpresa = (TextView) findViewById(R.id.textEmpresa);
    	textData = (TextView) findViewById(R.id.textData);
    	textDescripcio = (TextView) findViewById(R.id.textDescripcio);
    	textTotal = (TextView) findViewById(R.id.textTotal);
    	btnProductes = (ImageButton) findViewById(R.id.btnProductes);
    	btnProductesLayout = (LinearLayout) findViewById(R.id.btnProductesLayout);
    	btnOk = (ImageButton) findViewById(R.id.btnOk);
    	layoutFooter = (LinearLayout) findViewById(R.id.layoutFooter);
    	
    	// Assignem els valors
		if(empresa.getNom() != null) textEmpresa.setText(empresa.getNom());
	    String path = (empresa.getLogo()!=null) ? Utils.getAbsolutePath(empresa.getLogo()) : null;
	    if(path!=null && path.length()>0){
	        Bitmap bmp = BitmapFactory.decodeFile(path);
	        if(bmp != null) image.setImageBitmap(bmp);
	    }
    	
    	if(modeHistory){
    		if(comanda.getData() != null) textData.setText(comanda.getData());
    		if(comanda.getDescripcio() != null) textDescripcio.setText(comanda.getDescripcio());
    		
    		String total = CurrencyLocale.doubleToString(comanda.getTotal(), true);
    		textTotal.setText(total);
    		
    	    btnProductesLayout.setVisibility(LinearLayout.GONE);
    	    layoutFooter.setVisibility(LinearLayout.GONE);
    	}
    	else{ // Nova comanda
    		btnProductes.setOnClickListener(new View.OnClickListener() {
    			@Override
    			public void onClick(View v) {
    				if(quantitats != null){
    					quantitats.clear();
    					showQuantitats();
    				}
    				
					Intent intent = new Intent(ComandaEdicioActivity.this, ProducteLlistatActivity.class);
					intent.putExtra(ProducteLlistatActivity.EXTRA_MODECOMANDA, true);
					startActivityForResult(intent, SELECT_PRODUCTES);
    			}
    		});
    		
        	btnOk.setOnClickListener(new View.OnClickListener() {
    			@Override
    			public void onClick(View v) {
    				if(total <= 0){
    					dialogCampsBuits();
    				}
    				else{
    					insertComanda();
    				}
    			}
    		});
    	}
    	
    	showQuantitats();
    }
    
    @Override
	public void onActivityResult(int requestCode, int resultCode, Intent data) {
		super.onActivityResult(requestCode, resultCode, data); 
		
		Log.i("ComandaEdicioActivity.onActivityResult()");
		switch(requestCode) {
			case SELECT_PRODUCTES:
				if (resultCode == RESULT_OK) {
					quantitats = data.getParcelableArrayListExtra(ProducteLlistatActivity.EXTRA_QUANTITATS);
					showQuantitats();
				}
				break;
		}
	}  
    
    private void showQuantitats(){
    	if(quantitats!=null && textDescripcio!=null && textTotal!=null){
    		int id, num;
    		EntityProducte producte;
    		int size = quantitats.size();
    		
    		descripcio = "";
    		total = 0.0;
    		
    		for(int i=0; i<size; i++){
    			num = quantitats.get(i).quantitat;
    			id = quantitats.get(i).producte;
    			producte = productes.get(id);
    			if(producte != null){
    				descripcio += "- ("+num+") "+producte.getNom()+" ["+producte.getCodi()+"]\r\n";
    				total += producte.getPreu() * num;
    			}
    		}
    		
    		textDescripcio.setText(descripcio);
    		
    		String t = CurrencyLocale.doubleToString(total, true);
    		textTotal.setText(t);
    	}
    }
    
	public void dialogCampsBuits() {
		Log.i("Utils.dialogCampsBuits()"); 
		try{
			String msg = getString(R.string.comanda_omplir);
			
			AlertDialog.Builder builder = new AlertDialog.Builder(this);
			builder.setCancelable(false)
					.setMessage(msg)
					.setPositiveButton(getString(R.string.btn_ok), null);
			AlertDialog dialog = builder.create();
			dialog.setCancelable(false);
			dialog.show();
		}
		catch(Exception e){
			Log.e("/error:"+e.toString()); 
		}
	}
    
	private void insertComanda(){
		final EntityComanda httpEntity = new EntityComanda();
		httpEntity.setUsuari(user.getId());
		httpEntity.setEmpresa(empresa.getId());
		httpEntity.setDescripcio(descripcio);
		httpEntity.setTotal(total);
		httpEntity.setData(Utils.getCurrentDate());
		
		showLoading();
		
    	Utils.executeAsync(new Utils.AsyncListener() {
			@Override
			public boolean runBackground() {
				boolean ok = HttpManager.insertComanda(httpEntity);
				return ok;
			}
			
			@Override
			public void runPost(boolean ok) {
				hideLoading();
				if(ok){
					onBackPressed();
				}
				else{
					Utils.dialogHttpRestError(ComandaEdicioActivity.this);
				}
			}
		});
	}
	
	private void showLoading(){
		hideLoading();
		
		progressDialog = new ProgressDialog(this);
		progressDialog.setMessage(getString(R.string.loadingUpload));
		progressDialog.show();	
	}
	
	private void hideLoading(){
		if(progressDialog != null){
			progressDialog.dismiss();
			progressDialog = null;
		}
	}
}