package cat.cordecor.crm.activity;

import java.util.ArrayList;
import java.util.List;
import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.util.SparseArray;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ArrayAdapter;
import android.widget.EditText;
import android.widget.Filter;
import android.widget.Filterable;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.TextView;
import cat.cordecor.crm.R;
import cat.cordecor.crm.database.DataManager;
import cat.cordecor.crm.database.EntityComanda;
import cat.cordecor.crm.database.EntityEmpresa;
import cat.cordecor.crm.database.EntityUsuari;
import cat.cordecor.crm.util.CurrencyLocale;
import cat.cordecor.crm.util.Log;
import cat.cordecor.crm.util.Utils;

public class ComandaLlistatActivity extends android.support.v4.app.FragmentActivity implements OnItemClickListener{

	private SparseArray<EntityComanda> comandes;
	private SparseArray<EntityEmpresa> empreses;
	private RowAdapter adapter;
	private ListView lv;
	private EditText inputSearch;
	
	@Override
	public void onBackPressed() {
		super.onBackPressed();
		overridePendingTransition(R.anim.translate_right_in, R.anim.translate_right_out);
		finish();
	}
	
    @Override
    public void onCreate(Bundle savedInstanceState) {
    	super.onCreate(savedInstanceState);
    	Log.i("ComandaLlistatActivity.onCreate()");
    	
    	overridePendingTransition(R.anim.translate_left_in, R.anim.translate_left_out);
    	setContentView(R.layout.activity_comanda_llistat);
    	
    	Utils.configTopActivity(this, getString(R.string.menu_comandes));
    	
    	lv = (ListView) findViewById(R.id.listView);
    	lv.setOnItemClickListener(this);
    	
    	inputSearch = (EditText) findViewById(R.id.inputSearch);
    	
    	// Recuperem les dades per omplir el listview
    	Utils.executeAsync(new Utils.AsyncListener() {
			@Override
			public boolean runBackground() {
				empreses = DataManager.getInstance().getEmpreses();
				comandes = DataManager.getInstance().getComandes();
				return (comandes!=null) ? true : false;
			}
			
			@Override
			public void runPost(boolean ok) {
				configListView();
				configInputSearch();
			}
		});
    }   
    
    // Configurem les dades que li passarem a listview
    private void configListView(){
    	Log.i("ComandaLlistatActivity.configListView()");
    	
    	ArrayList<EntityHolder> list = new ArrayList<EntityHolder>();
    	EntityHolder entity;
		int size = (comandes!=null) ? comandes.size() : 0;
		
		for(int i=0; i<size; i++) {
			entity = new EntityHolder();
			entity.comanda = comandes.valueAt(i);
			if(empreses != null){
				entity.empresa = empreses.get(entity.comanda.getEmpresa());
			}
    		list.add(entity);
		}
		
		// Creem l'adaptador del listview
    	adapter = new RowAdapter(this, list);
        lv.setAdapter(adapter);
        lv.setTextFilterEnabled(true);
    }
    
    private void configInputSearch(){
    	// Controlem les recerques per filtrar els resultats del listview
    	inputSearch.addTextChangedListener(new TextWatcher() {
            
			@Override
			public void onTextChanged(CharSequence s, int start, int before, int count) {
				if(adapter != null){
					if (count < before) {
						// Estem esborrant carcters aix que hem de fer reset a l'adaptador
						adapter.resetData();
					}
					adapter.getFilter().filter(s.toString());
				}
			}
			
			@Override
			public void beforeTextChanged(CharSequence s, int start, int count, int after) {
			}
			
			@Override
			public void afterTextChanged(Editable s) {
			}
		});
    }
    
	@Override
	// Desprs de prmer sobre una fila del listview iniciarem la activity que mostri les dades de l'entitat
	public void onItemClick(AdapterView<?> av, View v, int position, long id) {
		EntityHolder entity = adapter.getItem(position);
		
		Intent intent = new Intent(this, ComandaEdicioActivity.class);
		intent.putExtra(ComandaEdicioActivity.EXTRA_COMANDA, entity.comanda);
		intent.putExtra(ComandaEdicioActivity.EXTRA_EMPRESA, entity.empresa);
		startActivity(intent);
	}
	
	public class EntityHolder{
		public EntityEmpresa empresa;
		public EntityComanda comanda;
	}
	
	// El nostre adaptador personalitzat per al listview
    public class RowAdapter extends ArrayAdapter<EntityHolder> implements Filterable {

    	private List<EntityHolder> rowList;
    	private List<EntityHolder> rowListOriginal;
    	private Context context;
    	private Filter rowFilter;
    	
    	public RowAdapter(Context ctx, List<EntityHolder> list) {
    		super(ctx, R.layout.activity_comanda_llistat_fila, list);
    		this.rowList = list;
    		this.rowListOriginal = list;
    		this.context = ctx;
    	}
    	
    	public int getCount() {
    		return rowList.size();
    	}

    	public EntityHolder getItem(int position) {
    		return rowList.get(position);
    	}

    	public long getItemId(int position) {
    		return rowList.get(position).hashCode();
    	}
    	
    	public void setItem(EntityHolder entity, int position){
    		EntityHolder entityOrig = rowList.get(position);
    		if(entityOrig != null){
    			entityOrig.empresa = entity.empresa;
    			entityOrig.comanda = entity.comanda;
    			notifyDataSetChanged();
    		}
		}
    	
    	public void addItem(EntityHolder entity){
    		if(entity != null){
    			rowListOriginal.add(entity);
    			resetData();
    			notifyDataSetChanged();
    		}
		}
    	
    	public View getView(int position, View convertView, ViewGroup parent) {
    		View v = convertView;
    		ViewHolder holder = new ViewHolder();
    		
    		// Verifiquem que el convertView no s nul
    		if (convertView == null) {
    			LayoutInflater inflater = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
    			v = inflater.inflate(R.layout.activity_comanda_llistat_fila, null);
    			
    	        holder.image = (ImageView) v.findViewById(R.id.image);
    	        holder.textTop = (TextView) v.findViewById(R.id.textTop);
    	        holder.textDown = (TextView) v.findViewById(R.id.textDown);
    	        holder.textRight = (TextView) v.findViewById(R.id.textRight);
    			
    			v.setTag(holder);
    		}
    		else{
    			holder = (ViewHolder) v.getTag();
    		}
    		
    		EntityHolder rowData = rowList.get(position);
    		
    		if(rowData.empresa != null){
        	    String path = (rowData.empresa.getLogo()!=null) ? Utils.getAbsolutePath(rowData.empresa.getLogo()) : null;
        	    Bitmap bmp = (path!=null && path.length()>0) ? BitmapFactory.decodeFile(path) : null;
        	    if(bmp != null){
        	        holder.image.setImageBitmap(bmp);
        	        holder.image.setVisibility(ImageView.VISIBLE);
        	    }
        	    else{
        	    	holder.image.setVisibility(ImageView.INVISIBLE);
        	    }
        	    
        	    if(rowData.empresa.getNom()!=null) holder.textTop.setText(rowData.empresa.getNom());
    		}
    		
    		if(rowData.comanda != null){
    			if(rowData.comanda.getData()!=null) holder.textDown.setText(rowData.comanda.getData());
    			
    			String total = CurrencyLocale.doubleToString(rowData.comanda.getTotal(), true);
    			holder.textRight.setText(total);
    		}
    		
    		return v;
    	}

    	public void resetData() {
    		rowList = rowListOriginal;
    	}
    	
    	private class ViewHolder {
    		public ImageView image;
    		public TextView textTop;
    		public TextView textDown;
    		public TextView textRight;
    	}
    	
    	@Override
    	public Filter getFilter() {
    		if (rowFilter == null)
    			rowFilter = new RowFilter();
    		
    		return rowFilter;
    	}

    	private class RowFilter extends Filter {

    		@Override
    		protected FilterResults performFiltering(CharSequence constraint) {
    			FilterResults results = new FilterResults();
    			
    			// Si no implementem el filtre tornem tota la llista
    			if (constraint == null || constraint.length() == 0) {
    				results.values = rowListOriginal;
    				results.count = rowListOriginal.size();
    			}
    			else {
    				List<EntityHolder> mList = new ArrayList<EntityHolder>();
    				
    				for (EntityHolder entity : rowList) {
    					if(entity.empresa!=null && entity.empresa.getNom().toLowerCase().contains(constraint.toString().toLowerCase()))
    					{
    						mList.add(entity);
    					}
    				}
    				
    				results.values = mList;
    				results.count = mList.size();
    			}
    			return results;
    		}

    		@Override
    		protected void publishResults(CharSequence constraint, FilterResults results) {
    			// Ara hem d'informar a l'adaptador que tenim una nova llista filtrada
				rowList = (List<EntityHolder>) results.values;
				notifyDataSetChanged();
    		}
    	}
    }
}