package cat.cordecor.crm.activity;

import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.drawable.BitmapDrawable;
import android.media.ThumbnailUtils;
import android.os.Bundle;
import android.view.View;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.LinearLayout;
import cat.cordecor.crm.R;
import cat.cordecor.crm.database.EntityContacte;
import cat.cordecor.crm.database.EntityUsuari;
import cat.cordecor.crm.http.HttpManager;
import cat.cordecor.crm.util.Log;
import cat.cordecor.crm.util.Utils;

public class ContacteEdicioActivity extends Activity {
	public static final String EXTRA_ENTITY = "entity";
	public static final String EXTRA_USER = "user";
	public static final String EXTRA_POSITION = "position";
	public static final String EXTRA_ACTION = "action";
	
	public static final int ACTION_INSERT = 0;
	public static final int ACTION_UPDATE = 1;
	public static final int ACTION_DELETE = 2;
	
	private static final int TAKE_PHOTO = 78;

	private ImageButton image;
	private EditText editNom;
	private EditText editCognoms;
	private EditText editTelefon;
	private EditText editEmail;
	private EditText editDireccio;
	private EditText editCiutat;
	private ImageButton btnEliminar;
	private ImageButton btnOk;
	private LinearLayout btnEliminarLayout;
	private ProgressDialog progressDialog = null;
	
	private EntityUsuari user;
	private EntityContacte entity;
	private int positionListview;
	private boolean modeEdition = false; // Si estem editant una Entitat existent o creem una nova
	private boolean imageChanged = false; // Si hem fet una nova foto
	
	@Override
	public void onBackPressed() {
		super.onBackPressed();
		overridePendingTransition(R.anim.translate_right_in, R.anim.translate_right_out);
		finish();
	}
	
    @Override
    public void onCreate(Bundle savedInstanceState) {
    	super.onCreate(savedInstanceState);
    	Log.i("ContacteEdicioActivity.onCreate()");
    	
    	overridePendingTransition(R.anim.translate_left_in, R.anim.translate_left_out);
    	setContentView(R.layout.activity_contacte_edicio);
    	
    	Utils.configTopActivity(this, getString(R.string.contacte_title));
    	
    	// Recuperem les dades que ens va passar la activity del listview
    	Intent intent = getIntent();
    	positionListview = intent.getIntExtra(EXTRA_POSITION, -1);
    	entity = intent.getParcelableExtra(EXTRA_ENTITY);
    	user = intent.getParcelableExtra(EXTRA_USER);
    	
    	if(entity != null){
    		Log.i("/toString(): "+entity.toString());
    		modeEdition = true;
    	}
    	else{
    		modeEdition = false;
    	}
    	
    	image = (ImageButton) findViewById(R.id.image);
    	editNom = (EditText) findViewById(R.id.editNom);
    	editCognoms = (EditText) findViewById(R.id.editCognoms);
    	editTelefon = (EditText) findViewById(R.id.editTelefon);
    	editEmail = (EditText) findViewById(R.id.editEmail);
    	editDireccio = (EditText) findViewById(R.id.editDireccio);
    	editCiutat = (EditText) findViewById(R.id.editCiutat);
    	btnEliminar = (ImageButton) findViewById(R.id.btnEliminar);
    	btnOk = (ImageButton) findViewById(R.id.btnOk);
    	btnEliminarLayout = (LinearLayout) findViewById(R.id.btnEliminarLayout);
    	
    	// Assignem els valors de l'entitat
    	if(modeEdition){
    		if(entity.getNom() != null) editNom.setText(entity.getNom());
    		if(entity.getCognoms() != null) editCognoms.setText(entity.getCognoms());
    		if(entity.getTelefon() != null) editTelefon.setText(entity.getTelefon());
    		if(entity.getEmail() != null) editEmail.setText(entity.getEmail());
    		if(entity.getDireccio() != null) editDireccio.setText(entity.getDireccio());
    		if(entity.getCiutat() != null) editCiutat.setText(entity.getCiutat());
    		
    	    String path = (entity.getFoto()!=null) ? Utils.getAbsolutePath(entity.getFoto()) : null;
    	    if(path!=null && path.length()>0){
    	        Bitmap bmp = BitmapFactory.decodeFile(path);
    	        if(bmp != null) image.setImageBitmap(bmp);
    	    }
    		
	    	btnEliminar.setOnClickListener(new View.OnClickListener() {
				@Override
				public void onClick(View v) {
					deleteEntity();
				}
			});
    	}
    	else{
    		// Si estem creant una nova entitat ocultem el bot eliminar
    		btnEliminarLayout.setVisibility(LinearLayout.GONE);
    	}
    	
    	image.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				startActivityForResult(new Intent(android.provider.MediaStore.ACTION_IMAGE_CAPTURE), TAKE_PHOTO);
			}
		});
    	
    	btnOk.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				if(editNom.getText().toString().isEmpty()){
					Utils.dialogCampsBuits(ContacteEdicioActivity.this, "Nom");
				}
				else{
					if(modeEdition){
						updateEntity();
					}
					else{
						insertEntity();
					}
				}
			}
		});
    }
    
	@Override
	public void onActivityResult(int requestCode, int resultCode, Intent data) {
		super.onActivityResult(requestCode, resultCode, data); 
		
		if (data == null)
			return;

		try{
			if (requestCode == TAKE_PHOTO){
				Bundle b = data.getExtras(); 
				Bitmap bmp = (Bitmap) b.get("data");
				if(bmp != null) bmp = ThumbnailUtils.extractThumbnail(bmp, 500, 500);

				if(bmp != null){
					imageChanged = true;
					image.setImageBitmap(bmp);
				}
			}
		}
		catch(Exception e){
			Log.e("ContacteEdicioActivity.onActivityResult(): "+e.toString());
		}
	}   
	
	private void insertEntity(){
		final EntityContacte httpEntity = new EntityContacte();
		if(user != null) httpEntity.setUsuari(user.getId());
		setValues(httpEntity);
		
		showLoading();
		
    	Utils.executeAsync(new Utils.AsyncListener() {
			@Override
			public boolean runBackground() {
				boolean ok = HttpManager.insertContacte(httpEntity);
				if(ok){
					boolean upload = uploadImage(httpEntity);
					if(upload) ok = HttpManager.updateContacte(httpEntity, upload);
				}
				return ok;
			}
			
			@Override
			public void runPost(boolean ok) {
				if(ok){
					goBack(httpEntity, positionListview, ACTION_INSERT);
				}
				else{
					Utils.dialogHttpRestError(ContacteEdicioActivity.this);
				}
				hideLoading();
			}
		});
	}
	
	private void updateEntity(){
		if(!valuesChanged()){
			Utils.dialogValorsNoModificats(this);
		}
		else{
			final EntityContacte httpEntity = entity.clone();
			setValues(httpEntity);
			
			showLoading();
			
	    	Utils.executeAsync(new Utils.AsyncListener() {
				@Override
				public boolean runBackground() {
					boolean upload = uploadImage(httpEntity);
					return HttpManager.updateContacte(httpEntity, upload);
				}
				
				@Override
				public void runPost(boolean ok) {
					if(ok){
						goBack(httpEntity, positionListview, ACTION_UPDATE);
					}
					else{
						Utils.dialogHttpRestError(ContacteEdicioActivity.this);
					}
					hideLoading();
				}
			});
		}
	}
	
	private void deleteEntity(){
		showLoading();
    	Utils.executeAsync(new Utils.AsyncListener() {
			@Override
			public boolean runBackground() {
				return HttpManager.deleteContacte(entity.getId());
			}
			
			@Override
			public void runPost(boolean ok) {
				if(ok){
					goBack(entity, positionListview, ACTION_DELETE);
				}
				else{
					Utils.dialogHttpRestError(ContacteEdicioActivity.this);
				}
				hideLoading();
			}
		});
	}
	
	private void showLoading(){
		hideLoading();
		
		progressDialog = new ProgressDialog(this);
		progressDialog.setMessage(getString(R.string.loadingUpload));
		progressDialog.show();	
	}
	
	private void hideLoading(){
		if(progressDialog != null){
			progressDialog.dismiss();
			progressDialog = null;
		}
	}
	
	// Comprovem si hem de pujar la imatge i si s aix assignem el nom que ha de tenir l'arxiu a l'entitat
	private boolean uploadImage(EntityContacte httpEntity){
		boolean uploadImage = false;
		if(imageChanged){
			Bitmap bmp = ((BitmapDrawable) image.getDrawable()).getBitmap();
			String fileName = EntityContacte.createImageFileName(httpEntity.getUsuari(), httpEntity.getId());
			if(bmp!=null && fileName!=null){
				String filePath = Utils.bitmapToLocalFile(bmp, fileName);
				if(filePath != null){
					httpEntity.setFoto(fileName);
					uploadImage = true;
				}
			}
		}
		return uploadImage;
	}
    
	// Tornem a la pantalla anterior assignant valors a Intent
    private void goBack(EntityContacte obj, int position, int mode){
		Intent intent = new Intent();
		if(position >= 0) intent.putExtra(EXTRA_POSITION, position);
		if(obj != null) intent.putExtra(EXTRA_ENTITY, obj);
		intent.putExtra(EXTRA_ACTION, mode);
		setResult(RESULT_OK,intent); 
		onBackPressed();
    }
    
	// Noms actualitzarem l'entitat si hem canviat algun dels seus valors
	private boolean valuesChanged(){
		boolean changed = false;
		if(!entity.getNom().equals( editNom.getText().toString()) ) changed = true;
		if(!entity.getCognoms().equals( editCognoms.getText().toString()) ) changed = true;
		if(!entity.getTelefon().equals( editTelefon.getText().toString()) ) changed = true;
		if(!entity.getEmail().equals( editEmail.getText().toString()) ) changed = true;
		if(!entity.getDireccio().equals( editDireccio.getText().toString()) ) changed = true;
		if(!entity.getCiutat().equals( editCiutat.getText().toString()) ) changed = true;
		return changed;
	}
	
	// Assignem els valors editables a l'entitat
	private void setValues(EntityContacte obj){
		obj.setNom( editNom.getText().toString() );
		obj.setCognoms( editCognoms.getText().toString() );
		obj.setTelefon( editTelefon.getText().toString() );
		obj.setEmail( editEmail.getText().toString() );
		obj.setDireccio( editDireccio.getText().toString() );
		obj.setCiutat( editCiutat.getText().toString() );
	}
}