package cat.cordecor.crm.activity;

import java.util.ArrayList;
import java.util.List;
import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.util.SparseArray;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ArrayAdapter;
import android.widget.EditText;
import android.widget.Filter;
import android.widget.Filterable;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.TextView;
import cat.cordecor.crm.R;
import cat.cordecor.crm.database.DataManager;
import cat.cordecor.crm.database.EntityEmpresa;
import cat.cordecor.crm.http.HttpImage;
import cat.cordecor.crm.util.Log;
import cat.cordecor.crm.util.Utils;

public class EmpresaLlistatActivity extends android.support.v4.app.FragmentActivity implements OnItemClickListener{

	private static final int ACTIVITY_DETAIL = 1;
	
	private SparseArray<EntityEmpresa> empreses;
	private RowAdapter adapter;
	private ListView lv;
	private EditText inputSearch;
	private ImageButton btnAfegir;
	
	@Override
	public void onBackPressed() {
		super.onBackPressed();
		overridePendingTransition(R.anim.translate_right_in, R.anim.translate_right_out);
		finish();
	}
	
    @Override
    public void onCreate(Bundle savedInstanceState) {
    	super.onCreate(savedInstanceState);
    	Log.i("EmpresaLlistatActivity.onCreate()");
    	
    	overridePendingTransition(R.anim.translate_left_in, R.anim.translate_left_out);
    	setContentView(R.layout.activity_contacte_llistat);
    	
    	Utils.configTopActivity(this, getString(R.string.menu_empreses));
    	
    	lv = (ListView) findViewById(R.id.listView);
    	lv.setOnItemClickListener(this);
    	
    	inputSearch = (EditText) findViewById(R.id.inputSearch);
    	
    	btnAfegir = (ImageButton) findViewById(R.id.btnAfegir);
    	btnAfegir.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				Intent intent = new Intent(EmpresaLlistatActivity.this, EmpresaEdicioActivity.class);
				startActivityForResult(intent, ACTIVITY_DETAIL);
			}
		});
    	
    	// Recuperem les dades per omplir el listview
    	Utils.executeAsync(new Utils.AsyncListener() {
			@Override
			public boolean runBackground() {
				empreses = DataManager.getInstance().getEmpreses();
				return (empreses != null) ? true : false;
			}
			
			@Override
			public void runPost(boolean ok) {
				configListView();
				configInputSearch();
			}
		});
    }   
    
    // Configurem les dades que li passarem a listview
    private void configListView(){
    	Log.i("EmpresaLlistatActivity.configListView()");
    	
    	ArrayList<EntityEmpresa> list = new ArrayList<EntityEmpresa>();
		EntityEmpresa entity;
		int size = (empreses!=null) ? empreses.size() : 0;
		
		for(int i=0; i<size; i++) {
			entity = empreses.valueAt(i);
    		list.add(entity);
		}
		
		// Creem l'adaptador del listview
    	adapter = new RowAdapter(this, list);
        lv.setAdapter(adapter);
        lv.setTextFilterEnabled(true);
    }
    
    private void configInputSearch(){
    	// Controlem les recerques per filtrar els resultats del listview
    	inputSearch.addTextChangedListener(new TextWatcher() {
            
			@Override
			public void onTextChanged(CharSequence s, int start, int before, int count) {
				if(adapter != null){
					if (count < before) {
						// Estem esborrant carcters aix que hem de fer reset a l'adaptador
						adapter.resetData();
					}
					adapter.getFilter().filter(s.toString());
				}
			}
			
			@Override
			public void beforeTextChanged(CharSequence s, int start, int count, int after) {
			}
			
			@Override
			public void afterTextChanged(Editable s) {
			}
		});
    }
    
	@Override
	// Desprs de prmer sobre una fila del listview iniciarem la activity que mostri les dades de l'entitat
	public void onItemClick(AdapterView<?> av, View v, int position, long id) {
		EntityEmpresa entity = adapter.getItem(position);
		
		Intent intent = new Intent(this, EmpresaEdicioActivity.class);
		intent.putExtra(EmpresaEdicioActivity.EXTRA_POSITION, position);
		intent.putExtra(EmpresaEdicioActivity.EXTRA_ENTITY, entity);
		startActivityForResult(intent, ACTIVITY_DETAIL);
	}
	
	@Override
	// Quan tornem a aquesta activity revisem si hem de modificar o esborrar una fila del listiview o afegir una nova
	protected void onActivityResult(int requestCode, int resultCode, Intent data){
		switch(requestCode) {
			case ACTIVITY_DETAIL:
				if (resultCode == RESULT_OK) {
					int action = data.getIntExtra(EmpresaEdicioActivity.EXTRA_ACTION, -1);
					int position = data.getIntExtra(EmpresaEdicioActivity.EXTRA_POSITION, -1);
					EntityEmpresa entity = data.getParcelableExtra(EmpresaEdicioActivity.EXTRA_ENTITY);
					
					if(entity!=null && adapter!=null && inputSearch!=null){
						// Afegim una fila
						if(action==EmpresaEdicioActivity.ACTION_INSERT){
							adapter.addItem(entity);
							inputSearch.setText("");
						}
						// Actualitzatem una fila
						else if(action==EmpresaEdicioActivity.ACTION_UPDATE && position>=0){
							adapter.setItem(entity, position);
						}
					}
				}
				break;
		}
	} 
    
	// El nostre adaptador personalitzat per al listview
    public class RowAdapter extends ArrayAdapter<EntityEmpresa> implements Filterable {

    	private List<EntityEmpresa> rowList;
    	private List<EntityEmpresa> rowListOriginal;
    	private Context context;
    	private Filter rowFilter;
    	
    	public RowAdapter(Context ctx, List<EntityEmpresa> list) {
    		super(ctx, R.layout.activity_empresa_llistat_fila, list);
    		this.rowList = list;
    		this.rowListOriginal = list;
    		this.context = ctx;
    	}
    	
    	public int getCount() {
    		return rowList.size();
    	}

    	public EntityEmpresa getItem(int position) {
    		return rowList.get(position);
    	}

    	public long getItemId(int position) {
    		return rowList.get(position).hashCode();
    	}
    	
    	public void setItem(EntityEmpresa entity, int position){
    		EntityEmpresa entityOrig = rowList.get(position);
    		if(entityOrig != null){
    			entityOrig.setValues(entity);
    			notifyDataSetChanged();
    		}
		}
    	
    	public void addItem(EntityEmpresa entity){
    		if(entity != null){
    			rowListOriginal.add(entity);
    			resetData();
    			notifyDataSetChanged();
    		}
		}
    	
    	public void removeItem(int position){
    		EntityEmpresa entity = rowList.remove(position);
    		if(entity != null) rowListOriginal.remove(entity);
    		notifyDataSetChanged();
		}

    	public View getView(int position, View convertView, ViewGroup parent) {
    		View v = convertView;
    		ViewHolder holder = new ViewHolder();
    		
    		// Verifiquem que el convertView no s nul
    		if (convertView == null) {
    			LayoutInflater inflater = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
    			v = inflater.inflate(R.layout.activity_empresa_llistat_fila, null);
    			
    	        holder.image = (ImageView) v.findViewById(R.id.image);
    	        holder.textTop = (TextView) v.findViewById(R.id.textTop);
    	        holder.textDown = (TextView) v.findViewById(R.id.textDown);
    	        holder.textRight = (TextView) v.findViewById(R.id.textRight);
    			
    			v.setTag(holder);
    		}
    		else{
    			holder = (ViewHolder) v.getTag();
    		}
    		
    		EntityEmpresa rowData = rowList.get(position);
    		
    		if(rowData.getNom()!=null) holder.textTop.setText(rowData.getNom());
    	    if(rowData.getDireccio()!=null) holder.textDown.setText(rowData.getDireccio());
    	    if(rowData.getEmail()!=null) holder.textRight.setText(rowData.getEmail());
    	    
    	    String path = (rowData.getLogo()!=null) ? Utils.getAbsolutePath(rowData.getLogo()) : null;
    	    Bitmap bmp = (path!=null && path.length()>0) ? BitmapFactory.decodeFile(path) : null;
    	    if(bmp != null){
    	        holder.image.setImageBitmap(bmp);
    	        holder.image.setVisibility(ImageView.VISIBLE);
    	    }
    	    else{
    	    	holder.image.setVisibility(ImageView.INVISIBLE);
    	    }
    		
    		return v;
    	}

    	public void resetData() {
    		rowList = rowListOriginal;
    	}
    	
    	private class ViewHolder {
    		public ImageView image;
    		public TextView textTop;
    		public TextView textDown;
    		public TextView textRight;
    	}
    	
    	@Override
    	public Filter getFilter() {
    		if (rowFilter == null)
    			rowFilter = new RowFilter();
    		
    		return rowFilter;
    	}

    	private class RowFilter extends Filter {

    		@Override
    		protected FilterResults performFiltering(CharSequence constraint) {
    			FilterResults results = new FilterResults();
    			
    			// Si no implementem el filtre tornem tota la llista
    			if (constraint == null || constraint.length() == 0) {
    				results.values = rowListOriginal;
    				results.count = rowListOriginal.size();
    			}
    			else {
    				List<EntityEmpresa> mList = new ArrayList<EntityEmpresa>();
    				
    				// Busquem les entitats que continguin en el seu nom o direcci el text escrit
    				for (EntityEmpresa entity : rowList) {
    					if(entity.getNom().toLowerCase().contains(constraint.toString().toLowerCase()) 
    							|| entity.getDireccio().toLowerCase().contains(constraint.toString().toLowerCase()))
    					{
    						mList.add(entity);
    					}
    				}
    				
    				results.values = mList;
    				results.count = mList.size();
    			}
    			return results;
    		}

    		@Override
    		protected void publishResults(CharSequence constraint, FilterResults results) {
    			// Ara hem d'informar a l'adaptador que tenim una nova llista filtrada
				rowList = (List<EntityEmpresa>) results.values;
				notifyDataSetChanged();
    		}
    	}
    }
}
