package cat.cordecor.crm.activity;

import java.io.File;

import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Intent;
import android.os.AsyncTask;
import android.os.Bundle;
import android.util.SparseArray;
import android.view.View;
import android.widget.EditText;
import android.widget.ImageButton;
import cat.cordecor.crm.App;
import cat.cordecor.crm.R;
import cat.cordecor.crm.database.DataManager;
import cat.cordecor.crm.database.EntityComanda;
import cat.cordecor.crm.database.EntityContacte;
import cat.cordecor.crm.database.EntityEmpresa;
import cat.cordecor.crm.database.EntityProducte;
import cat.cordecor.crm.database.EntityUsuari;
import cat.cordecor.crm.http.HttpImage;
import cat.cordecor.crm.http.HttpManager;
import cat.cordecor.crm.util.Log;
import cat.cordecor.crm.util.Utils;

public class LoginActivity extends Activity  {
	private static int STATE_READY = 0;
	private static int STATE_RUNNING = 1;
	
	private EntityUsuari user = null;
	private SparseArray<EntityEmpresa> empreses = null;
	private SparseArray<EntityContacte> contactes = null;
	private SparseArray<EntityProducte> productes = null;
	private SparseArray<EntityComanda> comandes = null;
	
	private String userEmail;
	private String userPassword;
	private LoginProgressDialog progressDialog;
	private AsyncRest asyncRest;
	private boolean ok;
	private int status;
	
	private ImageButton btnLogin;
	private EditText editEmail;
	private EditText editPassword;
	
	@Override
	public void onBackPressed(){
		super.onBackPressed();
		finish();
	}
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		
		Log.i("LoginActivity.onCreate()");
		status = STATE_READY;
		
		user = DataManager.getInstance().getUsuari();
		if(user != null){
			startRest(false);
		}
		else{
			setContentView(R.layout.activity_login);
			
			editEmail = (EditText) findViewById(R.id.editEmail);
			editPassword = (EditText) findViewById(R.id.editPassword);
			btnLogin = (ImageButton) findViewById(R.id.btnLogin);
			
			btnLogin.setOnClickListener(new View.OnClickListener() {
				
				@Override
				public void onClick(View v) {
					userEmail = editEmail.getText().toString();
					userPassword = editPassword.getText().toString();
					
					startRest(true);
				}
			});
		}
	}
	
    @Override
    public void onResume(){
    	super.onResume();  
    	
    	Log.i("LoginActivity.onResume()");
    	
		if (status==STATE_RUNNING && progressDialog!=null && !progressDialog.isShowing()){
			Log.i("/show progressDialog");
			progressDialog.show();
		}
    }
	
	private void startRest(boolean callLogin){
		Log.i("LoginActivity.startRest()");
		
		if (!Utils.isNetworkConnected(this)) {
			Log.i("/connection error");
		    Utils.exitDialog(LoginActivity.this, getString(R.string.checkConnection));
		}
		else if(status == STATE_READY){	 
			Log.i("/execute asyncRest");

	    	asyncRest = new AsyncRest();
	    	asyncRest.execute(callLogin);
		}		
	}
	
	private void startNextActivity() {
		Log.i("LoginActivity.startNextActivity()");
		
        Intent intent = new Intent(LoginActivity.this, MenuActivity.class);      
        intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
        startActivity(intent);
        finish();	
	}	
	
	private class LoginProgressDialog extends ProgressDialog{
		public LoginProgressDialog(Activity activity){
			super(activity);
		}
		
		@Override
		public void onBackPressed(){
			status = STATE_READY;
			if(asyncRest != null) asyncRest.cancel(true);
			this.dismiss();
			LoginActivity.this.onBackPressed();
		}
	}
	
	public class AsyncRest extends AsyncTask<Boolean, Integer, Boolean>{
		@Override
		protected void onPreExecute(){
			super.onPreExecute();
			
			Log.i("AsyncRest.onPreExecute()");			
			if(progressDialog == null){
				progressDialog = new LoginProgressDialog(LoginActivity.this);
				progressDialog.setCancelable(false);
				progressDialog.setMessage(getString(R.string.loginProgressDialog));
				progressDialog.setProgress(0);
				progressDialog.show();
			}
		}
		
		@Override
		protected Boolean doInBackground(Boolean... params) {
			Log.i("AsyncRest.doInBackground()");
			
			if(status == STATE_RUNNING){
				Log.i("AsyncRest /STATE_RUNNING");
				return false;
			}
			
			status = STATE_RUNNING;
			ok = true;
			boolean callLogin = (params.length>0) ? params[0] : true;

			// Usuari
			publishProgress(0);
			if(callLogin){
				user = HttpManager.login(userEmail, userPassword);
				if(user == null) ok = false;
			}
			else{
				user = HttpManager.getUsuari();
				if(user == null) ok = false;
			}
			
			// Empreses
			if(ok){
				publishProgress(1);
				empreses = HttpManager.getEmpreses();
				if(empreses == null) ok = false;
			}
			  
			// Contactes
			if(ok){
				publishProgress(2);
				contactes = HttpManager.getContactes();
				if(contactes == null) ok = false;
			}
			
			// Productes
			if(ok){
				publishProgress(3);
				productes = HttpManager.getProductes();
				if(productes == null) ok = false;
			}
			
			// Comandes
			if(ok){
				publishProgress(4);
				comandes = HttpManager.getComandes();
				if(comandes == null) ok = false;
			}
			
			// Imatges
			if(ok){
				publishProgress(5);
				downloadImages();
			}
			
			// REST finished
			if(ok){
				DataManager.getInstance().updateLoggedUser(user.getId());
				return true;
			}
			else{
				Log.i("AsyncRest /error");
				DataManager.getInstance().deleteLoggedUser();
				return false;
			}
		}
		
		@Override
		protected void onProgressUpdate(Integer... values) {
			super.onProgressUpdate(values);
			
			int statusProgress = (values.length>0) ? values[0] : 0;
			Log.i("AsyncRest.onProgressUpdate(): "+statusProgress);
           
			if(progressDialog != null){
				switch(statusProgress){
					case 0: progressDialog.setMessage(getString(R.string.loginProgressDialog));break;
					case 1: progressDialog.setMessage(getString(R.string.loginProgressDialogEmpreses));break;
					case 2: progressDialog.setMessage(getString(R.string.loginProgressDialogContactes));break;
					case 3: progressDialog.setMessage(getString(R.string.loginProgressDialogProductes));break;
					case 4: progressDialog.setMessage(getString(R.string.loginProgressDialogComandes));break;
					case 5: progressDialog.setMessage(getString(R.string.loginProgressDialogImatges));break;
				}
			}
		} 
		
		@Override
		protected void onPostExecute(Boolean result){	
			Log.i("AsyncRest.onPostExecute()");
			
			if (result == null) result = false;
			status = STATE_READY;
			
			if(progressDialog!=null && progressDialog.isShowing()){
				try{ 
					progressDialog.dismiss();
				}
				catch(Exception e){ }
			}
			    
			Log.i("/result: "+result);
			if(result){
				startNextActivity(); 
			}
			else{
				Utils.exitDialog(LoginActivity.this, getString(R.string.loginSyncError));
			}
		}	
	}	
	
	private void downloadImages() {
		int i,size;
		File file;
		String url;
		
		// Empreses
		EntityEmpresa empresa;
		size = empreses.size();
		for(i=0; i<size; i++) {
			empresa = empreses.valueAt(i);
			if(!empresa.getLogo().isEmpty()){
				url = HttpManager.DOWNLOAD_URL + empresa.getLogo();
				file = HttpImage.download(url, empresa.getLogo());
			}
		}
		
		// Contactes
		EntityContacte contacte;
		size = contactes.size();
		for(i=0; i<size; i++) {
			contacte = contactes.valueAt(i);
			if(!contacte.getFoto().isEmpty()){
				url = HttpManager.DOWNLOAD_URL + contacte.getFoto();
				file = HttpImage.download(url, contacte.getFoto());
			}
		}
		
		// Productes
		EntityProducte producte;
		size = productes.size();
		for(i=0; i<size; i++) {
			producte = productes.valueAt(i);
			if(!producte.getFoto().isEmpty()){
				url = HttpManager.DOWNLOAD_URL + producte.getFoto();
				file = HttpImage.download(url, producte.getFoto());
			}
		}
	}
}
