
package cat.cordecor.crm.activity;

import com.google.android.gms.maps.CameraUpdate;
import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.SupportMapFragment;
import com.google.android.gms.maps.model.CameraPosition;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.MarkerOptions;

import android.os.Bundle;
import android.util.SparseArray;
import cat.cordecor.crm.R;
import cat.cordecor.crm.database.DataManager;
import cat.cordecor.crm.database.EntityEmpresa;
import cat.cordecor.crm.util.Log;
import cat.cordecor.crm.util.Utils;

public class MapaActivity extends android.support.v4.app.FragmentActivity {

	private SparseArray<EntityEmpresa> empreses;
	private GoogleMap map;
	
	@Override
	public void onBackPressed() {
		super.onBackPressed();
		overridePendingTransition(R.anim.translate_right_in, R.anim.translate_right_out);
		finish();
	}
	
    @Override
    public void onCreate(Bundle savedInstanceState) {
    	super.onCreate(savedInstanceState);
    	Log.i("MapaActivity.onCreate()");
    	
    	overridePendingTransition(R.anim.translate_left_in, R.anim.translate_left_out);
    	setContentView(R.layout.activity_mapa);
    	
    	map = ((SupportMapFragment) getSupportFragmentManager().findFragmentById(R.id.map)).getMap();
    	
    	Utils.configTopActivity(this, getString(R.string.menu_mapa));
    	
    	// Recuperem les dades de les empreses
    	Utils.executeAsync(new Utils.AsyncListener() {
			@Override
			public boolean runBackground() {
				empreses = DataManager.getInstance().getEmpreses();
				return (empreses!=null) ? true : false;
			}
			
			@Override
			public void runPost(boolean ok) {
				configView();
			}
		});
    }
    
    private void configView(){
    	Log.i("MapaActivity.configView()");
    	
    	// Posici inicial
    	LatLng barcelona = new LatLng(41.39483307195529, 2.148767850000013);
    	CameraPosition camPos = new CameraPosition.Builder()
    	        .target(barcelona)   // Centrem el mapa a Barcelona
    	        .zoom(13)         
    	        //.bearing(45)      // Establim l'orientaci amb el nord-est a dalt
    	        .tilt(70)         // Baixem el punt de vista de la cmera 70 graus
    	        .build();
    	 
    	CameraUpdate camUpd3 =
    	    CameraUpdateFactory.newCameraPosition(camPos);
    	 
    	map.animateCamera(camUpd3);
    	
    	// Marcadors de les empreses
    	if(empreses != null){
    		EntityEmpresa empresa;
    		LatLng lng;
    		MarkerOptions marker;
    		int size = empreses.size();
    		
    		for(int i=0; i<size; i++){
    			empresa = empreses.get(i);
    			if(empresa!=null && empresa.getLatitud()!=0 && empresa.getLongitud()!=0){
    				lng = new LatLng(empresa.getLatitud(), empresa.getLongitud());
    				
    				marker = new MarkerOptions()
	    		        .position(lng)
	    		        .title(empresa.getNom());
    				
    				map.addMarker(marker);
    			}
    		}
    	}
    }
}