package cat.cordecor.crm.activity;

import cat.cordecor.crm.R;
import cat.cordecor.crm.database.DataManager;
import cat.cordecor.crm.database.EntityComanda;
import cat.cordecor.crm.database.EntityContacte;
import cat.cordecor.crm.database.EntityUsuari;
import cat.cordecor.crm.http.HttpManager;
import cat.cordecor.crm.util.Log;
import cat.cordecor.crm.util.Utils;
import android.os.AsyncTask;
import android.os.Bundle;
import android.util.SparseArray;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.EditText;
import android.widget.ImageButton;
import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Intent;

public class PerfilActivity extends Activity {

	private EntityUsuari user;
	
	private EditText editNom;
	private EditText editCognoms;
	private EditText editTelefon;
	private EditText editEmail;
	private EditText editPassword;
	private ImageButton btnOk;
	
	private ProgressDialog progressDialog = null;
	
	@Override
	public void onBackPressed() {
		super.onBackPressed();
		overridePendingTransition(R.anim.translate_right_in, R.anim.translate_right_out);
		finish();
	}
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		Log.i("PerfilActivity.onCreate()");
		
		overridePendingTransition(R.anim.translate_left_in, R.anim.translate_left_out);
		setContentView(R.layout.activity_perfil);
		
		Utils.configTopActivity(this, getString(R.string.menu_perfil));
		
    	Utils.executeAsync(new Utils.AsyncListener() {
			@Override
			public boolean runBackground() {
				user = DataManager.getInstance().getUsuari();
				return (user != null) ? true : false;
			}
			
			@Override
			public void runPost(boolean ok) {
				configView();
			}
		});
	}
	
    @Override
    protected void onResume() {
    	super.onResume();
    	Log.i("PerfilActivity.onResume()");
    }
    
	private void configView(){
		editNom = (EditText) findViewById(R.id.editNom);
		editCognoms = (EditText) findViewById(R.id.editCognoms);
		editTelefon = (EditText) findViewById(R.id.editTelefon);
		editEmail = (EditText) findViewById(R.id.editEmail);
		editPassword = (EditText) findViewById(R.id.editPassword);
		btnOk = (ImageButton) findViewById(R.id.btnOk);
		
		if(user != null){
			if(user.getNom() != null) editNom.setText(user.getNom());
			if(user.getCognoms() != null) editCognoms.setText(user.getCognoms());
			if(user.getTelefon() != null) editTelefon.setText(user.getTelefon());
			if(user.getEmail() != null) editEmail.setText(user.getEmail());
			if(user.getContrasenya() != null) editPassword.setText(user.getContrasenya());
			
			btnOk.setOnClickListener(new OnClickListener() {
				
				@Override
				public void onClick(View v) {
					if(user != null) updateEntity();
				}
			});
		}
		else{
			btnOk.setEnabled(false);
			btnOk.setVisibility(ImageButton.GONE);
		}
	}
	
	private void updateEntity(){
		if(!valuesChanged()){
			Utils.dialogValorsNoModificats(this);
		}
		else{
			final EntityUsuari httpEntity = user.clone();
			setValues(httpEntity);
			
			showLoading();
			
	    	Utils.executeAsync(new Utils.AsyncListener() {
				@Override
				public boolean runBackground() {
					return HttpManager.updateUsuari(httpEntity);
				}
				
				@Override
				public void runPost(boolean ok) {
					if(ok){
						onBackPressed();
					}
					else{
						Utils.dialogHttpRestError(PerfilActivity.this);
					}
					hideLoading();
				}
			});
		}
	}
	
	private void showLoading(){
		hideLoading();
		
		progressDialog = new ProgressDialog(this);
		progressDialog.setMessage(getString(R.string.loadingUpload));
		progressDialog.show();	
	}
	
	private void hideLoading(){
		if(progressDialog != null){
			progressDialog.dismiss();
			progressDialog = null;
		}
	}
	
	// Noms actualitzarem l'entitat si hem canviat algun dels seus valors
	private boolean valuesChanged(){
		boolean changed = false;
		if(!user.getNom().equals( editNom.getText().toString()) ) changed = true;
		if(!user.getCognoms().equals( editCognoms.getText().toString()) ) changed = true;
		if(!user.getTelefon().equals( editTelefon.getText().toString()) ) changed = true;
		return changed;
	}
	
	// Assignem els valors editables a l'entitat
	private void setValues(EntityUsuari obj){
		obj.setNom( editNom.getText().toString() );
		obj.setCognoms( editCognoms.getText().toString() );
		obj.setTelefon( editTelefon.getText().toString() );
	}
}
