package cat.cordecor.crm.activity;

import java.util.ArrayList;
import java.util.List;

import android.app.Dialog;
import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Bundle;
import android.os.Parcel;
import android.os.Parcelable;
import android.os.Parcelable.Creator;
import android.text.Editable;
import android.text.TextWatcher;
import android.util.SparseArray;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Filter;
import android.widget.Filterable;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.NumberPicker;
import android.widget.TextView;
import cat.cordecor.crm.R;
import cat.cordecor.crm.database.DataManager;
import cat.cordecor.crm.database.EntityComanda;
import cat.cordecor.crm.database.EntityContacte;
import cat.cordecor.crm.database.EntityEmpresa;
import cat.cordecor.crm.database.EntityProducte;
import cat.cordecor.crm.http.HttpImage;
import cat.cordecor.crm.util.CurrencyLocale;
import cat.cordecor.crm.util.Log;
import cat.cordecor.crm.util.Utils;

public class ProducteLlistatActivity extends android.support.v4.app.FragmentActivity implements OnItemClickListener{

	public static final String EXTRA_MODECOMANDA = "mode";
	public static final String EXTRA_QUANTITATS = "quantitats";
	
	private SparseArray<EntityProducte> productes;
	private RowAdapter adapter;
	private ListView lv;
	private EditText inputSearch;
	private ImageButton btnOk;
	private LinearLayout layoutFooter;
	
	private boolean modeComanda; // Si estem seleccionant quantitats per una comanda
	private SparseArray<Integer> quantitats;
	
	@Override
	public void onBackPressed() {
		super.onBackPressed();
		overridePendingTransition(R.anim.translate_right_in, R.anim.translate_right_out);
		finish();
	}
	
    @Override
    public void onCreate(Bundle savedInstanceState) {
    	super.onCreate(savedInstanceState);
    	Log.i("ProducteLlistatActivity.onCreate()");
    	
    	overridePendingTransition(R.anim.translate_left_in, R.anim.translate_left_out);
    	setContentView(R.layout.activity_producte_llistat);
    	
    	Utils.configTopActivity(this, getString(R.string.menu_productes));
    	
    	// Recuperem les dades que ens va passar la activity del listview
    	Intent intent = getIntent();
    	modeComanda = intent.getBooleanExtra(EXTRA_MODECOMANDA, false);
    	
    	// Vista
    	btnOk = (ImageButton) findViewById(R.id.btnOk);
    	layoutFooter = (LinearLayout) findViewById(R.id.layoutFooter);
    	
    	lv = (ListView) findViewById(R.id.listView);
    	lv.setOnItemClickListener(this);
    	
    	inputSearch = (EditText) findViewById(R.id.inputSearch);
    	
    	if(modeComanda){
    		quantitats = new SparseArray<Integer>();
    		
        	btnOk.setOnClickListener(new View.OnClickListener() {
    			@Override
    			public void onClick(View v) {
    				goBack();
    			}
    		});
        	
        	layoutFooter.setVisibility(LinearLayout.VISIBLE);
    	}
    	
    	// Recuperem les dades per omplir el listview
    	Utils.executeAsync(new Utils.AsyncListener() {
			@Override
			public boolean runBackground() {
				productes = DataManager.getInstance().getProductes();
				return (productes != null) ? true : false;
			}
			
			@Override
			public void runPost(boolean ok) {
				configListView();
				configInputSearch();
			}
		});
    }   
    
    // Configurem les dades que li passarem a listview
    private void configListView(){
    	Log.i("ProducteLlistatActivity.configListView()");
    	
    	ArrayList<EntityHolder> list = new ArrayList<EntityHolder>();
    	EntityHolder entity;
		int size = (productes!=null) ? productes.size() : 0;
		
		for(int i=0; i<size; i++) {
			entity = new EntityHolder();
			entity.producte = productes.valueAt(i);
    		list.add(entity);
		}
		
		// Creem l'adaptador del listview
    	adapter = new RowAdapter(this, list);
        lv.setAdapter(adapter);
        lv.setTextFilterEnabled(true);
    }
    
    private void configInputSearch(){
    	// Controlem les recerques per filtrar els resultats del listview
    	inputSearch.addTextChangedListener(new TextWatcher() {
            
			@Override
			public void onTextChanged(CharSequence s, int start, int before, int count) {
				if(adapter != null){
					if (count < before) {
						// Estem esborrant carcters aix que hem de fer reset a l'adaptador
						adapter.resetData();
					}
					adapter.getFilter().filter(s.toString());
				}
			}
			
			@Override
			public void beforeTextChanged(CharSequence s, int start, int count, int after) {
			}
			
			@Override
			public void afterTextChanged(Editable s) {
			}
		});
    }
    
	@Override
	// Desprs de prmer sobre una fila del listview iniciarem la activity que mostri les dades de l'entitat
	public void onItemClick(AdapterView<?> av, View v, int position, long id) {
		EntityHolder holder = adapter.getItem(position);
		
		if(modeComanda){
			showDialogQuantity(holder, position);
		}
		else{
			Intent intent = new Intent(this, ProducteDetallActivity.class);
			intent.putExtra(ProducteDetallActivity.EXTRA_ENTITY, holder.producte);
			startActivity(intent);
		}
	}
	
	public class EntityHolder{
		public EntityProducte producte;
		public int quantitat = 0;
	}
	
	// El nostre adaptador personalitzat per al listview
    public class RowAdapter extends ArrayAdapter<EntityHolder> implements Filterable {

    	private List<EntityHolder> rowList;
    	private List<EntityHolder> rowListOriginal;
    	private Context context;
    	private Filter rowFilter;
    	
    	public RowAdapter(Context ctx, List<EntityHolder> list) {
    		super(ctx, R.layout.activity_producte_llistat_fila, list);
    		this.rowList = list;
    		this.rowListOriginal = list;
    		this.context = ctx;
    	}
    	
    	public int getCount() {
    		return rowList.size();
    	}

    	public EntityHolder getItem(int position) {
    		return rowList.get(position);
    	}

    	public long getItemId(int position) {
    		return rowList.get(position).hashCode();
    	}
    	
    	public void setItem(EntityHolder entity, int position){
    		EntityHolder entityOrig = rowList.get(position);
    		if(entityOrig != null){
    			entityOrig.producte = entity.producte;
    			entityOrig.quantitat = entity.quantitat;
    			notifyDataSetChanged();
    		}
		}
    	
    	public View getView(int position, View convertView, ViewGroup parent) {
    		View v = convertView;
    		ViewHolder holder = new ViewHolder();
    		
    		// Verifiquem que el convertView no s nul
    		if (convertView == null) {
    			LayoutInflater inflater = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
    			v = inflater.inflate(R.layout.activity_producte_llistat_fila, null);
    			
    	        holder.image = (ImageView) v.findViewById(R.id.image);
    	        holder.name = (TextView) v.findViewById(R.id.textViewName);
    	        holder.description = (TextView) v.findViewById(R.id.textViewDescription);
    	        holder.price = (TextView) v.findViewById(R.id.textViewPrice);
    	        holder.amount = (TextView) v.findViewById(R.id.viewAmount);
    			
    			v.setTag(holder);
    		}
    		else{
    			holder = (ViewHolder) v.getTag();
    		}
    		
    		EntityHolder rowData = rowList.get(position);
    		EntityProducte producte = rowData.producte;
    		
    		if(producte.getNom()!=null) holder.name.setText(producte.getNom());
    	    if(producte.getDescripcio()!=null) holder.description.setText(producte.getDescripcio());

    		String t = CurrencyLocale.doubleToString(producte.getPreu(), true);
    		holder.price.setText(t);
    	    
    	    String path = (producte.getFoto()!=null) ? Utils.getAbsolutePath(producte.getFoto()) : null;
    	    Bitmap bmp = (path!=null && path.length()>0) ? BitmapFactory.decodeFile(path) : null;
    	    if(bmp != null){
    	        holder.image.setImageBitmap(bmp);
    	        holder.image.setVisibility(ImageView.VISIBLE);
    	    }
    	    else{
    	    	holder.image.setVisibility(ImageView.INVISIBLE);
    	    }
    	    
    	    if(modeComanda){
    	    	holder.amount.setVisibility(TextView.VISIBLE);
    	    	holder.amount.setText(Integer.toString(rowData.quantitat));
    	    }
    		
    		return v;
    	}

    	public void resetData() {
    		rowList = rowListOriginal;
    	}
    	
    	private class ViewHolder {
    		public ImageView image;
    		public TextView name;
    		public TextView description;
    		public TextView price;
    		public TextView amount;
    	}
    	
    	@Override
    	public Filter getFilter() {
    		if (rowFilter == null)
    			rowFilter = new RowFilter();
    		
    		return rowFilter;
    	}

    	private class RowFilter extends Filter {

    		@Override
    		protected FilterResults performFiltering(CharSequence constraint) {
    			FilterResults results = new FilterResults();
    			
    			// Si no implementem el filtre tornem tota la llista
    			if (constraint == null || constraint.length() == 0) {
    				results.values = rowListOriginal;
    				results.count = rowListOriginal.size();
    			}
    			else {
    				List<EntityHolder> mList = new ArrayList<EntityHolder>();
    				
    				// Busquem les entitats que continguin en el seu nom o cognoms el text escrit
    				for (EntityHolder entity : rowList) {
    					if(entity.producte.getNom().toLowerCase().contains(constraint.toString().toLowerCase()))
    					{
    						mList.add(entity);
    					}
    				}
    				
    				results.values = mList;
    				results.count = mList.size();
    			}
    			return results;
    		}

    		@Override
    		protected void publishResults(CharSequence constraint, FilterResults results) {
    			// Ara hem d'informar a l'adaptador que tenim una nova llista filtrada
				rowList = (List<EntityHolder>) results.values;
				notifyDataSetChanged();
    		}
    	}
    }
    
    // Comanda ---------------------------
    
    private void showDialogQuantity(final EntityHolder holder, final int position){
		final Dialog d = new Dialog(this);
		d.setTitle(getString(R.string.producte_dialog_title));
		d.setContentView(R.layout.dialog_quantity);
		
		Button btnCancelar = (Button) d.findViewById(R.id.btnCancelar);
		Button btnOK = (Button) d.findViewById(R.id.btnOK);
		
		final NumberPicker np = (NumberPicker) d.findViewById(R.id.numberPicker);
		np.setMaxValue(100); // valor mxim 100
		np.setMinValue(0); // valor mnim 0
		np.setWrapSelectorWheel(false);
		
		btnCancelar.setOnClickListener(new Button.OnClickListener() {
			@Override
			public void onClick(View v) {
				d.dismiss(); 
			}
		});
		btnOK.setOnClickListener(new Button.OnClickListener() {
			@Override
			public void onClick(View v) {
				if(adapter != null){
					holder.quantitat = np.getValue();
					adapter.setItem(holder, position);
					
					// Associem l'id del producte amb la quantitat seleccionada
					quantitats.put(holder.producte.getId(), holder.quantitat);
				}

				d.dismiss();
			}
		});
		d.show();
    }
    
    private void goBack(){
    	ArrayList<ProducteQuantitat> list = new ArrayList<ProducteQuantitat>();
    	ProducteQuantitat p;
		int size = (quantitats!=null) ? quantitats.size() : 0;
		for(int i=0; i<size; i++) {
			if(quantitats.valueAt(i) > 0){ // les quantitats ms grans que 0
				p = new ProducteQuantitat();
				p.producte = quantitats.keyAt(i);
				p.quantitat = quantitats.valueAt(i);
				
				list.add(p);
			}
		}
    	
		Intent intent = new Intent();
		intent.putExtra(EXTRA_QUANTITATS, list);
		setResult(RESULT_OK, intent); 
		onBackPressed();
    }
    
    public static class ProducteQuantitat implements Parcelable {
    	public int producte;
    	public int quantitat;
    	
        public ProducteQuantitat(){}

        public ProducteQuantitat(Parcel in){
        	producte = in.readInt();
        	quantitat = in.readInt();
    	}	

        public void writeToParcel(Parcel dest, int flags) {
        	dest.writeInt(producte);
            dest.writeInt(quantitat);
        }	
        
        public int describeContents() {
            return 0;
        }    
        
        public static final Creator<ProducteQuantitat> CREATOR = new Creator<ProducteQuantitat>() {
            public ProducteQuantitat createFromParcel(Parcel in) {
                return new ProducteQuantitat(in);
            }

            public ProducteQuantitat[] newArray(int size) {
                return new ProducteQuantitat[size];
            }
        };
    }
}