package cat.cordecor.crm.database;

import java.util.concurrent.atomic.AtomicInteger;
import android.database.sqlite.SQLiteDatabase;
import android.os.AsyncTask;
import android.util.SparseArray;
import cat.cordecor.crm.App;
import cat.cordecor.crm.util.Log;

public class DataManager {

    private static DataManager instance;
    
    private DataSqlite mDatabaseHelper;
    private SQLiteDatabase mDatabase;
    private AtomicInteger mOpenCounter = new AtomicInteger();

    private DataManager() {
        mDatabaseHelper = new DataSqlite(App.getAppContext());
    }

    public static synchronized DataManager getInstance() {
        if (instance == null) {
        	instance = new DataManager();
        }
        return instance;
    }
   
    private synchronized SQLiteDatabase openDatabase() {
    	if(mDatabaseHelper == null) mDatabaseHelper = new DataSqlite(App.getAppContext());
    	if(mOpenCounter == null) mOpenCounter = new AtomicInteger();
    	
    	if(mOpenCounter.incrementAndGet() == 1){
            mDatabase = mDatabaseHelper.getWritableDatabase();
        }
        Log.d("Database open counter: " + mOpenCounter.get());
        return mDatabase;
    }

    private synchronized void closeDatabase() {
    	if(mOpenCounter == null) mOpenCounter = new AtomicInteger();
    	
    	if(mOpenCounter.decrementAndGet()==0 && mDatabase!=null) {
            mDatabase.close();
        }
        Log.d("Database open counter: " + mOpenCounter.get());
    }
    
    // Usuari ----------------------------------------------------------
    public void updateLoggedUser(int id){
        SQLiteDatabase database = openDatabase();
        DataSqlite.updateLoggedUser(database, id);
        closeDatabase();	
	}
    
    public void deleteLoggedUser(){
        SQLiteDatabase database = openDatabase();
        DataSqlite.deleteLoggedUser(database);
        closeDatabase();	
	}
    
    public EntityUsuari getUsuari(){
    	if(DataRuntime.getUsuari() == null){
            SQLiteDatabase database = openDatabase();
            DataRuntime.setUsuari( DataSqlite.getLoggedUser(database) );
            closeDatabase();
    	}
    	return DataRuntime.getUsuari();
    }
    
    public boolean modifyUsuari(EntityUsuari entity){
    	if(entity != null){
            SQLiteDatabase database = openDatabase();
            boolean ok = EntityUsuari.databaseUpdateInsert(database, entity);
			closeDatabase();
			if(ok){
				DataRuntime.setUsuari(entity);
			}
    		return ok;
    	}
    	else{
    		return false;
    	}
    }
    
    // Empreses -------------------------------------------------------
    public SparseArray<EntityEmpresa> getEmpreses(){
    	EntityUsuari user = getUsuari();
    	if(DataRuntime.getEmpreses()==null && user!=null){
            SQLiteDatabase database = openDatabase();
            DataRuntime.setEmpreses( EntityEmpresa.databaseGetAll(database) );
			closeDatabase();
    	}
    	return DataRuntime.getEmpreses();
    }
    
    public boolean modifyEmpresa(EntityEmpresa entity){
    	if(entity != null){
            SQLiteDatabase database = openDatabase();
            boolean ok = EntityEmpresa.databaseUpdateInsert(database, entity);
			closeDatabase();
			if(ok){
				DataRuntime.updateInsertEmpresa(entity);
			}
    		return ok;
    	}
    	else{
    		return false;
    	}
    }
    
    public boolean deleteEmpresa(int id){
        SQLiteDatabase database = openDatabase();
        boolean ok = EntityEmpresa.databaseDelete(database, id);
		closeDatabase();
		if(ok){
			DataRuntime.deleteEmpresa(id);
		}
		return ok;
    }
    
    // Contactes --------------------------------------------------------
    public SparseArray<EntityContacte> getContactes(){
    	EntityUsuari user = getUsuari();
    	if(DataRuntime.getContactes()==null && user!=null){
            SQLiteDatabase database = openDatabase();
            DataRuntime.setContactes( EntityContacte.databaseGetAll(database, user.getId()) );
			closeDatabase();
    	}
    	return DataRuntime.getContactes();
    }
    
    public boolean modifyContacte(EntityContacte entity){
    	if(entity != null){
    		Log.i("DataManager.modifyContacte() /id:"+entity.getId());
            SQLiteDatabase database = openDatabase();
            boolean ok = EntityContacte.databaseUpdateInsert(database, entity);
			closeDatabase();
			if(ok){
				DataRuntime.updateInsertContacte(entity);
			}
    		return ok;
    	}
    	else{
    		return false;
    	}
    }
    
    public boolean deleteContacte(int id){
    	Log.i("DataManager.deleteContacte() /id:"+id);
        SQLiteDatabase database = openDatabase();
        boolean ok = EntityContacte.databaseDelete(database, id);
		closeDatabase();
		if(ok){
			DataRuntime.deleteContacte(id);
		}
		return ok;
    }
    
    // Productes -------------------------------------------------------
    public SparseArray<EntityProducte> getProductes(){
    	EntityUsuari user = getUsuari();
    	if(DataRuntime.getProductes()==null && user!=null){
            SQLiteDatabase database = openDatabase();
            DataRuntime.setProductes( EntityProducte.databaseGetAll(database) );
			closeDatabase();
    	}
    	return DataRuntime.getProductes();
    }
    
    public boolean modifyProducte(EntityProducte entity){
    	if(entity != null){
            SQLiteDatabase database = openDatabase();
            boolean ok = EntityProducte.databaseUpdateInsert(database, entity);
			closeDatabase();
			if(ok){
				DataRuntime.updateInsertProducte(entity);
			}
    		return ok;
    	}
    	else{
    		return false;
    	}
    }
    
    public boolean deleteProducte(int id){
        SQLiteDatabase database = openDatabase();
        boolean ok = EntityProducte.databaseDelete(database, id);
		closeDatabase();
		if(ok){
			DataRuntime.deleteProducte(id);
		}
		return ok;
    }
    
    // Comandes --------------------------------------------------------
    public SparseArray<EntityComanda> getComandes(){
    	EntityUsuari user = getUsuari();
    	if(DataRuntime.getComandes()==null && user!=null){
            SQLiteDatabase database = openDatabase();
            DataRuntime.setComandes( EntityComanda.databaseGetAll(database, user.getId()) );
			closeDatabase();
    	}
    	return DataRuntime.getComandes();
    }
    
    public boolean modifyComanda(EntityComanda entity){
    	if(entity != null){
            SQLiteDatabase database = openDatabase();
            boolean ok = EntityComanda.databaseUpdateInsert(database, entity);
			closeDatabase();
			if(ok){
				DataRuntime.updateInsertComanda(entity);
			}
    		return ok;
    	}
    	else{
    		return false;
    	}
    }
}
