package cat.cordecor.crm.database;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;

import cat.cordecor.crm.util.Log;

public class DataSqlite extends SQLiteOpenHelper{
	
	private static final String DATABASE_NAME = "cordecor.sqlite";
	private static final int DATABASE_VERSION = 1;
	
    private static final String LOGGEDUSER_TABLE_NAME = "loggedUser";
    private static final String LOGGEDUSER_COLUMN_ID = "id";
    
    public DataSqlite(Context context) {
        super(context, DATABASE_NAME, null, DATABASE_VERSION);
    }

    @Override
    public void onCreate(SQLiteDatabase db) {
    	Log.i("DataSqlite.onCreate()");
    	
        // create all tables
    	db.execSQL(EntityComanda.sqlCreateTable());
    	db.execSQL(EntityContacte.sqlCreateTable());
    	db.execSQL(EntityEmpresa.sqlCreateTable());
    	db.execSQL(EntityProducte.sqlCreateTable());
    	db.execSQL(EntityUsuari.sqlCreateTable());
    	db.execSQL("CREATE TABLE "+LOGGEDUSER_TABLE_NAME+"("+LOGGEDUSER_COLUMN_ID+" INTEGER)");
    }

    @Override
    public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
        if (newVersion > oldVersion) {
            // drop all tables
        	db.execSQL(EntityComanda.sqlDropTable());
        	db.execSQL(EntityContacte.sqlDropTable());
        	db.execSQL(EntityEmpresa.sqlDropTable());
        	db.execSQL(EntityProducte.sqlDropTable());
        	db.execSQL(EntityUsuari.sqlDropTable());
        	db.execSQL("DROP TABLE IF EXISTS "+LOGGEDUSER_TABLE_NAME);
            
            //re-create all tables
            onCreate(db);
        }
    }
    
    // ---------------------------------------------------------------
    
    public static EntityUsuari getLoggedUser(SQLiteDatabase db){
        if(db == null)
        	return null;

        int id;
        EntityUsuari entity = null;
		try{
		    String sql = "SELECT * FROM "+LOGGEDUSER_TABLE_NAME;
		    Cursor c = db.rawQuery(sql, new String[] {});
		    
		    int count = 0;
		    for(c.moveToFirst(); !c.isAfterLast() && count==0; c.moveToNext()){
		    	if(c.getColumnIndex(LOGGEDUSER_COLUMN_ID)>=0){
		    		id = c.getInt(c.getColumnIndex(LOGGEDUSER_COLUMN_ID));
		    		entity = EntityUsuari.databaseGet(db, id);
		    	}
		    	count++;
		    }
		}
		catch(Exception e){
			Log.e("DataManagerSqlite.getLoggedUser(): "+e.toString());
			entity = null;
		}
        return entity;
	}
    
    public static boolean updateLoggedUser(SQLiteDatabase db, int id){
        if(db == null)
                return false;

        deleteLoggedUser(db);
        
        ContentValues values = new ContentValues();
        values.put(LOGGEDUSER_COLUMN_ID, id);

    	long row = db.insert(LOGGEDUSER_TABLE_NAME, null, values);
    	Log.i("DataSqlite.updateLoggedUser() /row: "+row);
    	
    	if(row >= 0) return true;
    	else return false;
	}
    
    public static void deleteLoggedUser(SQLiteDatabase db){
        if(db != null){
        	int total = db.delete(LOGGEDUSER_TABLE_NAME, null, null);
        	Log.i("DataSqlite.deleteLoggedUser() /total: "+total);
        }
	}
}
