package cat.cordecor.crm.database;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import cat.cordecor.crm.http.RestResponse;
import cat.cordecor.crm.util.Log;
import android.content.ContentValues;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.os.Parcel;
import android.os.Parcelable;
import android.util.SparseArray;

public class EntityComanda implements Parcelable {

	// Database Table
	public static final String TABLE_NAME = "comanda";

	// Database Columns
	public static final String COLUMN_ID = "id";
	public static final String COLUMN_USUARI = "usuari"; // Cada comanda est associada a l'usuari que l'ha creat
	public static final String COLUMN_EMPRESA = "empresa";
	public static final String COLUMN_DESCRIPCIO = "descripcio";
	public static final String COLUMN_TOTAL = "total";
	public static final String COLUMN_DATA = "data";

	private int id;
	private int usuari;
	private int empresa;
	private String descripcio;
	private double total;
	private String data;

	public EntityComanda() {
	}

	public EntityComanda(Parcel in) {
		id = in.readInt();
		usuari = in.readInt();
		empresa = in.readInt();
		descripcio = in.readString();
		total = in.readDouble();
		data = in.readString();
	}

	// Parcel
	public void writeToParcel(Parcel dest, int flags) {
		dest.writeInt(id);
		dest.writeInt(usuari);
		dest.writeInt(empresa);
		dest.writeString(descripcio);
		dest.writeDouble(total);
		dest.writeString(data);
	}

	public int describeContents() {
		return 0;
	}

	public static final Creator<EntityComanda> CREATOR = new Creator<EntityComanda>() {
		public EntityComanda createFromParcel(Parcel in) {
			return new EntityComanda(in);
		}

		public EntityComanda[] newArray(int size) {
			return new EntityComanda[size];
		}
	};

	// Getters & Setters
	public int getId() {
		return id;
	}

	public int getUsuari() {
		return usuari;
	}

	public int getEmpresa() {
		return empresa;
	}

	public String getDescripcio() {
		return descripcio;
	}

	public double getTotal() {
		return total;
	}
	
	public String getData() {
		return data;
	}

	public void setId(int value) {
		id = value;
	}

	public void setUsuari(int value) {
		usuari = value;
	}

	public void setEmpresa(int value) {
		empresa = value;
	}

	public void setDescripcio(String value) {
		descripcio = value;
	}

	public void setTotal(double value) {
		total = value;
	}
	
	public void setData(String value) {
		data = value;
	}

    public void setValues(EntityComanda entity){
    	if(entity != null){
			id = entity.getId();
			usuari = entity.getUsuari();
			empresa = entity.getEmpresa();
			descripcio = entity.getDescripcio();
			total = entity.getTotal();
			data = entity.getData();
    	}
    }
    
    public EntityComanda clone(){
    	EntityComanda entity = new EntityComanda();
    	entity.setValues(this);
    	return entity;
    }
	
	@Override
	public String toString() {
		return "EntityComanda [id=" + id + ", usuari=" + usuari + ", empresa="
				+ empresa + ", descripcio=" + descripcio + ", total=" + total + ", data=" + data
				+ "]";
	}

	// JSON
	public static String toJSON(EntityComanda entity) {
		try {
			JSONObject json = new JSONObject();
			json.put(COLUMN_ID, entity.getId());
			json.put(COLUMN_USUARI, entity.getUsuari());
			json.put(COLUMN_EMPRESA, entity.getEmpresa());
			json.put(COLUMN_DESCRIPCIO, entity.getDescripcio());
			json.put(COLUMN_TOTAL, entity.getTotal());
			json.put(COLUMN_DATA, entity.getData());

			return json.toString();
		} catch (JSONException e) {
			e.printStackTrace();
		}
		return null;
	}

    public static EntityComanda parseJSONEntity(JSONObject obj) throws JSONException{
    	EntityComanda entity = new EntityComanda();
		entity.setId(obj.getInt(COLUMN_ID));
		entity.setUsuari(obj.getInt(COLUMN_USUARI));
		entity.setEmpresa(obj.getInt(COLUMN_EMPRESA));
		entity.setDescripcio(obj.getString(COLUMN_DESCRIPCIO));
		entity.setTotal(obj.getDouble(COLUMN_TOTAL));
		entity.setData(obj.getString(COLUMN_DATA));
        return entity;
    }
    
    public static SparseArray<EntityComanda> parseHttpGetAll(String jsonStr) {
    	Log.i("EntityComanda.parseHttpGetAll()"); 
    	SparseArray<EntityComanda> data = new SparseArray<EntityComanda>();
    	EntityComanda entity;
        try {
             JSONObject json = new JSONObject(jsonStr);
             JSONArray list = json.getJSONArray(RestResponse.TAG_RESULT);
             JSONObject obj;
             
             int size = list.length();
             for (int i=0; i<size; i++) {
            	 obj = list.getJSONObject(i);
                 entity = parseJSONEntity(obj);
                 data.put(entity.getId(), entity);
                 
                 DataManager.getInstance().modifyComanda(entity); // Modifica en DataManager
             }
        }
        catch (Exception e) {
        	data = null;
            e.printStackTrace();
        }
        return data;
    }
    
    public static SparseArray<EntityComanda> parseHttpGetSync(String jsonStr) {
    	Log.i("EntityComanda.parseHttpGetSync()"); 
    	SparseArray<EntityComanda> data = new SparseArray<EntityComanda>();
    	EntityComanda entity;
        try {
             JSONObject json = new JSONObject(jsonStr);
             json = json.getJSONObject(RestResponse.TAG_RESULT);
             JSONArray jsonMod = json.getJSONArray(RestResponse.TAG_RESULT_MODIFIED);
             JSONObject obj;
             int i,size;
             
             // Modificats
             size = jsonMod.length();
             for (i=0; i<size; i++) {
            	 obj = jsonMod.getJSONObject(i);
                 entity = parseJSONEntity(obj);
                 data.put(entity.getId(), entity);
                 
                 DataManager.getInstance().modifyComanda(entity); // Modifica en DataManager
             }
        }
        catch (Exception e) {
        	data = null;
            e.printStackTrace();
        }
        return data;
    }
    

	// Database -------------------------------------------------
	public static String sqlCreateTable() {
		String sql = "CREATE TABLE " + TABLE_NAME + " (" + COLUMN_ID
				+ " INTEGER NOT NULL, " + COLUMN_USUARI + " INTEGER NOT NULL, "
				+ COLUMN_EMPRESA + " INTEGER NOT NULL, " + COLUMN_DESCRIPCIO
				+ " TEXT, " + COLUMN_TOTAL + " REAL, " + COLUMN_DATA + " TEXT, " + "PRIMARY KEY ("
				+ COLUMN_ID + "));";
		return sql;
	}

	public static String sqlDropTable() {
		return "DROP TABLE IF EXISTS " + TABLE_NAME;
	}
	
    public static SparseArray<EntityComanda> databaseGetAll(SQLiteDatabase db, int usuari){
        if(db == null)
        	return null;

        SparseArray<EntityComanda> data = new SparseArray<EntityComanda>();
		try{
			EntityComanda entity;
		    String sql = "SELECT * FROM "+TABLE_NAME+" WHERE usuari="+usuari;
		    Cursor cursor = db.rawQuery(sql, new String[] {});
		    
		    for(cursor.moveToFirst(); !cursor.isAfterLast(); cursor.moveToNext()){
		    	entity = fromCursor(cursor);
		    	data.append(entity.getId(), entity);
		    }
		}
		catch(Exception e){
			Log.e("EntityComanda.databaseGet(): "+e.toString());
		}
        return data;
	}

    public static boolean databaseUpdateInsert(SQLiteDatabase db, EntityComanda entity){
        if(db==null || entity==null)
                return false;

        ContentValues values = new ContentValues();
		values.put(COLUMN_USUARI, entity.getUsuari());
		values.put(COLUMN_EMPRESA, entity.getEmpresa());
		values.put(COLUMN_DESCRIPCIO, entity.getDescripcio());
		values.put(COLUMN_TOTAL, entity.getTotal());
		values.put(COLUMN_DATA, entity.getData());

        int rows = db.update(TABLE_NAME, values, COLUMN_ID+"="+entity.getId(), null);
        if(rows > 0){
        	return true;
        }
        else{
        	values.put(COLUMN_ID, entity.getId());
        	long row = db.insert(TABLE_NAME, null, values);
        	
        	if(row >= 0) return true;
        	else return false;
        }
	}	
    
    public static EntityComanda fromCursor(Cursor c) {
    	EntityComanda entity = new EntityComanda();

		if(c.getColumnIndex(COLUMN_ID)>=0) entity.setId( c.getInt(c.getColumnIndex(COLUMN_ID)) );
		if(c.getColumnIndex(COLUMN_USUARI)>=0) entity.setUsuari( c.getInt(c.getColumnIndex(COLUMN_USUARI)) );
        if(c.getColumnIndex(COLUMN_EMPRESA)>=0) entity.setEmpresa( c.getInt(c.getColumnIndex(COLUMN_EMPRESA)) );
        if(c.getColumnIndex(COLUMN_DESCRIPCIO)>=0) entity.setDescripcio( c.getString(c.getColumnIndex(COLUMN_DESCRIPCIO)) );
        if(c.getColumnIndex(COLUMN_TOTAL)>=0) entity.setTotal( c.getDouble(c.getColumnIndex(COLUMN_TOTAL)) );
        if(c.getColumnIndex(COLUMN_DATA)>=0) entity.setData( c.getString(c.getColumnIndex(COLUMN_DATA)) );

        return entity;
    }
}
