package cat.cordecor.crm.database;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import cat.cordecor.crm.http.RestResponse;
import cat.cordecor.crm.util.Log;
import android.content.ContentValues;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.os.Parcel;
import android.os.Parcelable;
import android.util.SparseArray;

public class EntityContacte implements Parcelable {

	// Database Table
    public static final String TABLE_NAME = "contacte";
	
	// Database Columns
    public static final String COLUMN_ID = "id";
    public static final String COLUMN_USUARI = "usuari"; // Cada contacte est associat a l'usuari que l'ha creat
    public static final String COLUMN_NOM = "nom";
    public static final String COLUMN_COGNOMS = "cognoms";
    public static final String COLUMN_DIRECCIO = "direccio";
    public static final String COLUMN_CIUTAT = "ciutat";
    public static final String COLUMN_TELEFON = "telefon";
    public static final String COLUMN_EMAIL = "email";
    public static final String COLUMN_FOTO = "foto";
	
	private int id;
	private int usuari;
    private String nom;
    private String cognoms;
    private String direccio;
    private String ciutat;
    private String telefon;
    private String email;
    private String foto;

    public EntityContacte(){}

    // Parcel
    public EntityContacte(Parcel in){
		id = in.readInt();
		usuari = in.readInt();
        nom = in.readString();
        cognoms = in.readString();
        direccio = in.readString();
        ciutat = in.readString();
        telefon = in.readString();
        email = in.readString();
        foto = in.readString();
	}	
    
    public void writeToParcel(Parcel dest, int flags) {
    	dest.writeInt(id);
    	dest.writeInt(usuari);
        dest.writeString(nom);
        dest.writeString(cognoms);
        dest.writeString(direccio);
        dest.writeString(ciutat);
        dest.writeString(telefon);
        dest.writeString(email);
        dest.writeString(foto);
    }	
    
    public int describeContents() {
        return 0;
    }    
    
    public static final Creator<EntityContacte> CREATOR = new Creator<EntityContacte>() {
        public EntityContacte createFromParcel(Parcel in) {
            return new EntityContacte(in);
        }

        public EntityContacte[] newArray(int size) {
            return new EntityContacte[size];
        }
    };    

    // Getters & Setters
    public int getId(){ return id; }
    public int getUsuari(){ return usuari; }
    public String getNom(){ return nom; }
    public String getCognoms(){ return cognoms; }
    public String getDireccio(){ return direccio; }
    public String getCiutat(){ return ciutat; }
    public String getTelefon(){ return telefon; }
    public String getEmail(){ return email; }
    public String getFoto(){ return foto; }

    public void setId(int value){ id = value; }
    public void setUsuari(int value){ usuari = value; }
    public void setNom(String value){ nom = value; }
    public void setCognoms(String value){ cognoms = value; }
    public void setDireccio(String value){ direccio = value; }
    public void setCiutat(String value){ ciutat = value; }
    public void setTelefon(String value){ telefon = value; }
    public void setEmail(String value){ email = value; }
    public void setFoto(String value){ foto = value; }

    public void setValues(EntityContacte entity){
    	if(entity != null){
			id = entity.getId();
			usuari = entity.getUsuari();
	        nom = entity.getNom();
	        cognoms = entity.getCognoms();
	        direccio = entity.getDireccio();
	        ciutat = entity.getCiutat();
	        telefon = entity.getTelefon();
	        email = entity.getEmail();
	        foto = entity.getFoto();
    	}
    }
    
    public EntityContacte clone(){
    	EntityContacte entity = new EntityContacte();
    	entity.setValues(this);
    	return entity;
    }
    
    @Override
    public String toString() {
            return "EntityContacte [id=" + id
            				+ ", usuari=" + usuari
                            + ", nom=" + nom
                            + ", cognoms=" + cognoms
                            + ", direccio=" + direccio
                            + ", ciutat=" + ciutat
                            + ", telefon=" + telefon
                            + ", email=" + email
                            + ", foto=" + foto
                            + "]";
    }
    
    public static String createImageFileName(int idUsuari, int idContacte){
    	String file = null;
    	if(idUsuari>0 && idContacte>0) file = "contacte_u"+idUsuari+"_"+idContacte+".jpg";
    	return file;
    }

    // JSON
    public static String toJSON(EntityContacte entity) {
        try {
            JSONObject json = new JSONObject();
            json.put(COLUMN_ID, entity.getId());
            json.put(COLUMN_USUARI, entity.getUsuari());
            json.put(COLUMN_NOM, entity.getNom());
            json.put(COLUMN_COGNOMS, entity.getCognoms());
            json.put(COLUMN_DIRECCIO, entity.getDireccio());
            json.put(COLUMN_CIUTAT, entity.getCiutat());
            json.put(COLUMN_TELEFON, entity.getTelefon());
            json.put(COLUMN_EMAIL, entity.getEmail());
            json.put(COLUMN_FOTO, entity.getFoto());

            return json.toString();
        }
        catch(JSONException e) {
            e.printStackTrace();
        }
        return null;
    }
    
    public static EntityContacte parseJSONEntity(JSONObject obj) throws JSONException{
    	EntityContacte entity = new EntityContacte();
        entity.setId(obj.getInt(COLUMN_ID));
        entity.setUsuari(obj.getInt(COLUMN_USUARI));
        entity.setNom(obj.getString(COLUMN_NOM));
        entity.setCognoms(obj.getString(COLUMN_COGNOMS));
        entity.setDireccio(obj.getString(COLUMN_DIRECCIO));
        entity.setCiutat(obj.getString(COLUMN_CIUTAT));
        entity.setTelefon(obj.getString(COLUMN_TELEFON));
        entity.setEmail(obj.getString(COLUMN_EMAIL));
        entity.setFoto(obj.getString(COLUMN_FOTO));
        return entity;
    }
    
    public static SparseArray<EntityContacte> parseHttpGetAll(String jsonStr) {
    	Log.i("EntityContacte.parseHttpGetAll()"); 
    	SparseArray<EntityContacte> data = new SparseArray<EntityContacte>();
        EntityContacte entity;
        try {
             JSONObject json = new JSONObject(jsonStr);
             JSONArray list = json.getJSONArray(RestResponse.TAG_RESULT);
             JSONObject obj;
             
             int size = list.length();
             for (int i=0; i<size; i++) {
            	 obj = list.getJSONObject(i);
                 entity = parseJSONEntity(obj);
                 data.put(entity.getId(), entity);
                 
                 DataManager.getInstance().modifyContacte(entity); // Modifica en DataManager
             }
        }
        catch (Exception e) {
        	data = null;
            e.printStackTrace();
        }
        return data;
    }
    
    public static SparseArray<EntityContacte> parseHttpGetSync(String jsonStr) {
    	Log.i("EntityContacte.parseHttpGetSync()"); 
    	SparseArray<EntityContacte> data = new SparseArray<EntityContacte>();
        EntityContacte entity;
        try {
             JSONObject json = new JSONObject(jsonStr);
             json = json.getJSONObject(RestResponse.TAG_RESULT);
             JSONArray jsonDel = json.getJSONArray(RestResponse.TAG_RESULT_DELETED);
             JSONArray jsonMod = json.getJSONArray(RestResponse.TAG_RESULT_MODIFIED);
             JSONObject obj;
             int i,size,id;
             
             // Eliminats
             size = jsonDel.length();
             for (i=0; i<size; i++) {
            	 obj = jsonDel.getJSONObject(i);
            	 id = obj.getInt(COLUMN_ID);

                 DataManager.getInstance().deleteContacte(id); // Elimina en DataManager
             }
             
             // Modificats
             size = jsonMod.length();
             for (i=0; i<size; i++) {
            	 obj = jsonMod.getJSONObject(i);
                 entity = parseJSONEntity(obj);
                 data.put(entity.getId(), entity);
                 
                 DataManager.getInstance().modifyContacte(entity); // Modifica en DataManager
             }
        }
        catch (Exception e) {
        	data = null;
            e.printStackTrace();
        }
        return data;
    }
    
    // Database -------------------------------------------------
	public static String sqlCreateTable(){
                String sql = "CREATE TABLE "+TABLE_NAME+" ("
                + COLUMN_ID + " INTEGER NOT NULL, "
                + COLUMN_USUARI + " INTEGER NOT NULL, "
                + COLUMN_NOM + " TEXT, "
                + COLUMN_COGNOMS + " TEXT, "
                + COLUMN_DIRECCIO + " TEXT, "
                + COLUMN_CIUTAT + " TEXT, "
                + COLUMN_TELEFON + " TEXT, "
                + COLUMN_EMAIL + " TEXT, "
                + COLUMN_FOTO + " TEXT, "
                + "PRIMARY KEY (" + COLUMN_ID + "));";
		return sql;
	}
	
	public static String sqlDropTable(){
                return "DROP TABLE IF EXISTS "+TABLE_NAME;
	}
	
    public static SparseArray<EntityContacte> databaseGetAll(SQLiteDatabase db, int usuari){
    	Log.i("EntityContacte.databaseGetAll()");
        if(db == null)
        	return null;

        SparseArray<EntityContacte> data = new SparseArray<EntityContacte>();
		try{
			EntityContacte entity;
		    String sql = "SELECT * FROM "+TABLE_NAME+" WHERE "+COLUMN_USUARI+"="+usuari;
		    Cursor cursor = db.rawQuery(sql, new String[] {});
		    Log.i("/cursor size:"+cursor.getCount());
		    
		    for(cursor.moveToFirst(); !cursor.isAfterLast(); cursor.moveToNext()){
		    	entity = fromCursor(cursor);
		    	data.append(entity.getId(), entity);
		    }
		    Log.i("/data size:"+data.size());
		}
		catch(Exception e){
			Log.e("EntityContacte.databaseGet(): "+e.toString());
		}
        return data;
	}
	
    public static boolean databaseUpdateInsert(SQLiteDatabase db, EntityContacte entity){
    	Log.i("EntityContacte.databaseUpdateInsert()");
        if(db==null || entity==null)
                return false;

        ContentValues values = new ContentValues();
        values.put(COLUMN_USUARI, entity.getUsuari());
        values.put(COLUMN_NOM, entity.getNom());
        values.put(COLUMN_COGNOMS, entity.getCognoms());
        values.put(COLUMN_DIRECCIO, entity.getDireccio());
        values.put(COLUMN_CIUTAT, entity.getCiutat());
        values.put(COLUMN_TELEFON, entity.getTelefon());
        values.put(COLUMN_EMAIL, entity.getEmail());
        values.put(COLUMN_FOTO, entity.getFoto());

        int rows = db.update(TABLE_NAME, values, COLUMN_ID+"="+entity.getId(), null);
        Log.i("/updated rows:"+rows);
        
        if(rows > 0){
        	return true;
        }
        else{
        	values.put(COLUMN_ID, entity.getId());
        	long row = db.insert(TABLE_NAME, null, values);
        	Log.i("/inserted row:"+row);
        	
        	if(row >= 0) return true;
        	else return false;
        }
	}
    
    public static boolean databaseDelete(SQLiteDatabase db, int id){
    	Log.i("EntityContacte.databaseDelete()");
        if(db == null)
        	return false;
        
        int rows = db.delete(TABLE_NAME, COLUMN_ID+"="+id, null);
        Log.i("/deleted rows:"+rows);
        if(rows > 0){
        	return true;
        }
        else{ 
        	return false;
        }
	}
    
    public static EntityContacte fromCursor(Cursor c) {
    	EntityContacte entity = new EntityContacte();

		if(c.getColumnIndex(COLUMN_ID)>=0) entity.setId( c.getInt(c.getColumnIndex(COLUMN_ID)) );
		if(c.getColumnIndex(COLUMN_USUARI)>=0) entity.setUsuari( c.getInt(c.getColumnIndex(COLUMN_USUARI)) );
		if(c.getColumnIndex(COLUMN_NOM)>=0) entity.setNom( c.getString(c.getColumnIndex(COLUMN_NOM)) );
        if(c.getColumnIndex(COLUMN_COGNOMS)>=0) entity.setCognoms( c.getString(c.getColumnIndex(COLUMN_COGNOMS)) );
        if(c.getColumnIndex(COLUMN_DIRECCIO)>=0) entity.setDireccio( c.getString(c.getColumnIndex(COLUMN_DIRECCIO)) );
        if(c.getColumnIndex(COLUMN_CIUTAT)>=0) entity.setCiutat( c.getString(c.getColumnIndex(COLUMN_CIUTAT)) );
        if(c.getColumnIndex(COLUMN_TELEFON)>=0) entity.setTelefon( c.getString(c.getColumnIndex(COLUMN_TELEFON)) );
        if(c.getColumnIndex(COLUMN_EMAIL)>=0) entity.setEmail( c.getString(c.getColumnIndex(COLUMN_EMAIL)) );
        if(c.getColumnIndex(COLUMN_FOTO)>=0) entity.setFoto( c.getString(c.getColumnIndex(COLUMN_FOTO)) );

        return entity;
    }
}
