package cat.cordecor.crm.database;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import cat.cordecor.crm.http.RestResponse;
import cat.cordecor.crm.util.Log;
import android.content.ContentValues;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.os.Parcel;
import android.os.Parcelable;
import android.util.SparseArray;

public class EntityEmpresa implements Parcelable {

	// Database Table
    public static final String TABLE_NAME = "empresa";
	
	// Database Columns
    public static final String COLUMN_ID = "id";
    public static final String COLUMN_NOM = "nom";
    public static final String COLUMN_DIRECCIO = "direccio";
    public static final String COLUMN_CIUTAT = "ciutat";
    public static final String COLUMN_TELEFON = "telefon";
    public static final String COLUMN_EMAIL = "email";
    public static final String COLUMN_LOGO = "logo";
    public static final String COLUMN_LATITUD = "latitud";
    public static final String COLUMN_LONGITUD = "longitud";    
	
	private int id;
    private String nom;
    private String direccio;
    private String ciutat;
    private String telefon;
    private String email;
    private String logo;
    private double latitud;
    private double longitud;

    public EntityEmpresa(){}

    public EntityEmpresa(Parcel in){
    	id = in.readInt();
        nom = in.readString();
        direccio = in.readString();
        ciutat = in.readString();
        telefon = in.readString();
        email = in.readString();
        logo = in.readString();
        latitud = in.readDouble();
        longitud = in.readDouble();
	}	

    // Parcel
    public void writeToParcel(Parcel dest, int flags) {
    	dest.writeInt(id);
        dest.writeString(nom);
        dest.writeString(direccio);
        dest.writeString(ciutat);
        dest.writeString(telefon);
        dest.writeString(email);
        dest.writeString(logo);
        dest.writeDouble(latitud);
        dest.writeDouble(longitud);
    }	
    
    public int describeContents() {
        return 0;
    }    
    
    public static final Creator<EntityEmpresa> CREATOR = new Creator<EntityEmpresa>() {
        public EntityEmpresa createFromParcel(Parcel in) {
            return new EntityEmpresa(in);
        }

        public EntityEmpresa[] newArray(int size) {
            return new EntityEmpresa[size];
        }
    };    

    // Getters & Setters
    public int getId(){ return id; }
    public String getNom(){ return nom; }
    public String getDireccio(){ return direccio; }
    public String getCiutat(){ return ciutat; }
    public String getTelefon(){ return telefon; }
    public String getEmail(){ return email; }
    public String getLogo(){ return logo; }
    public double getLatitud(){ return latitud; }
    public double getLongitud(){ return longitud; }

    public void setId(int value){ id = value; }
    public void setNom(String value){ nom = value; }
    public void setDireccio(String value){ direccio = value; }
    public void setCiutat(String value){ ciutat = value; }
    public void setTelefon(String value){ telefon = value; }
    public void setEmail(String value){ email = value; }
    public void setLogo(String value){ logo = value; }
    public void setLatitud(double value){ latitud = value; }
    public void setLongitud(double value){ longitud = value; }

    public void setValues(EntityEmpresa entity){
    	if(entity != null){
			id = entity.getId();
			nom = entity.getNom();
			direccio = entity.getDireccio();
			ciutat = entity.getCiutat();
			telefon = entity.getTelefon();
			email = entity.getEmail();
			logo = entity.getLogo();
			latitud = entity.getLatitud();
			longitud = entity.getLongitud();
    	}
    }
    
    public EntityEmpresa clone(){
    	EntityEmpresa entity = new EntityEmpresa();
    	entity.setValues(this);
    	return entity;
    }
    
    @Override
    public String toString() {
            return "EntityEmpresa [id=" + id
                            + ", nom=" + nom
                            + ", direccio=" + direccio
                            + ", ciutat=" + ciutat
                            + ", telefon=" + telefon
                            + ", email=" + email
                            + ", logo=" + logo
                            + ", latitud=" + latitud
                            + ", longitud=" + longitud
                            + "]";
    }

    public static String createImageFileName(int idEmpresa){
    	String file = null;
    	if(idEmpresa>0) file = "empresa_"+idEmpresa+".jpg";
    	return file;
    }

    
    // JSON
    public static String toJSON(EntityEmpresa entity) {
        try {
            JSONObject json = new JSONObject();
            json.put(COLUMN_ID, entity.getId());
            json.put(COLUMN_NOM, entity.getNom());
            json.put(COLUMN_DIRECCIO, entity.getDireccio());
            json.put(COLUMN_CIUTAT, entity.getCiutat());
            json.put(COLUMN_TELEFON, entity.getTelefon());
            json.put(COLUMN_EMAIL, entity.getEmail());
            json.put(COLUMN_LOGO, entity.getLogo());
            json.put(COLUMN_LATITUD, entity.getLatitud());
            json.put(COLUMN_LONGITUD, entity.getLongitud());

            return json.toString();
        }
        catch(JSONException e) {
            e.printStackTrace();
        }
        return null;
    }

    public static EntityEmpresa parseJSONEntity(JSONObject obj) throws JSONException{
    	EntityEmpresa entity = new EntityEmpresa();
        entity.setId(obj.getInt(COLUMN_ID));
        entity.setNom(obj.getString(COLUMN_NOM));
        entity.setDireccio(obj.getString(COLUMN_DIRECCIO));
        entity.setCiutat(obj.getString(COLUMN_CIUTAT));
        entity.setTelefon(obj.getString(COLUMN_TELEFON));
        entity.setEmail(obj.getString(COLUMN_EMAIL));
        entity.setLogo(obj.getString(COLUMN_LOGO));
        entity.setLatitud(obj.getDouble(COLUMN_LATITUD));
        entity.setLongitud(obj.getDouble(COLUMN_LONGITUD));
        return entity;
    }
    
    public static SparseArray<EntityEmpresa> parseHttpGetAll(String jsonStr) {
    	Log.i("EntityEmpresa.parseHttpGetAll()"); 
    	SparseArray<EntityEmpresa> data = new SparseArray<EntityEmpresa>();
    	EntityEmpresa entity;
        try {
             JSONObject json = new JSONObject(jsonStr);
             JSONArray list = json.getJSONArray(RestResponse.TAG_RESULT);
             JSONObject obj;
             
             int size = list.length();
             for (int i=0; i<size; i++) {
            	 obj = list.getJSONObject(i);
                 entity = parseJSONEntity(obj);
                 data.put(entity.getId(), entity);
                 
                 DataManager.getInstance().modifyEmpresa(entity); // Modifica en DataManager
             }
        }
        catch (Exception e) {
        	data = null;
            e.printStackTrace();
        }
        return data;
    }
    
    public static SparseArray<EntityEmpresa> parseHttpGetSync(String jsonStr) {
    	Log.i("EntityEmpresa.parseHttpGetSync()"); 
    	SparseArray<EntityEmpresa> data = new SparseArray<EntityEmpresa>();
    	EntityEmpresa entity;
        try {
             JSONObject json = new JSONObject(jsonStr);
             json = json.getJSONObject(RestResponse.TAG_RESULT);
             JSONArray jsonDel = json.getJSONArray(RestResponse.TAG_RESULT_DELETED);
             JSONArray jsonMod = json.getJSONArray(RestResponse.TAG_RESULT_MODIFIED);
             JSONObject obj;
             int i,size,id;
               
             // Eliminats
             size = jsonDel.length();
             for (i=0; i<size; i++) {
            	 obj = jsonDel.getJSONObject(i);
            	 id = obj.getInt(COLUMN_ID);

                 DataManager.getInstance().deleteEmpresa(id); // Elimina en DataManager
             }
             
             // Modificats
             size = jsonMod.length();
             for (i=0; i<size; i++) {
            	 obj = jsonMod.getJSONObject(i);
                 entity = parseJSONEntity(obj);
                 data.put(entity.getId(), entity);
                 
                 DataManager.getInstance().modifyEmpresa(entity); // Modifica en DataManager
             }
        }
        catch (Exception e) {
        	data = null;
            e.printStackTrace();
        }
        return data;
    }

    // Database -------------------------------------------------
	public static String sqlCreateTable(){
                String sql = "CREATE TABLE "+TABLE_NAME+" ("
                + COLUMN_ID + " INTEGER NOT NULL, "
                + COLUMN_NOM + " TEXT, "
                + COLUMN_DIRECCIO + " TEXT, "
                + COLUMN_CIUTAT + " TEXT, "
                + COLUMN_TELEFON + " TEXT, "
                + COLUMN_EMAIL + " TEXT, "
                + COLUMN_LOGO + " TEXT, "
                + COLUMN_LATITUD + " REAL, "
                + COLUMN_LONGITUD + " REAL, "
                + "PRIMARY KEY (" + COLUMN_ID + "));";
                Log.i(":::::"+sql);
		return sql;
	}
	
	public static String sqlDropTable(){
                return "DROP TABLE IF EXISTS "+TABLE_NAME;
	}
	
    public static SparseArray<EntityEmpresa> databaseGetAll(SQLiteDatabase db){
        if(db == null)
        	return null;

        SparseArray<EntityEmpresa> data = new SparseArray<EntityEmpresa>();
		try{
			EntityEmpresa entity;
		    String sql = "SELECT * FROM "+TABLE_NAME;
		    Cursor cursor = db.rawQuery(sql, new String[] {});
		    
		    for(cursor.moveToFirst(); !cursor.isAfterLast(); cursor.moveToNext()){
		    	entity = fromCursor(cursor);
		    	data.append(entity.getId(), entity);
		    }
		}
		catch(Exception e){
			Log.e("EntityEmpresa.databaseGet(): "+e.toString());
		}
        return data;
	}
	
    public static boolean databaseUpdateInsert(SQLiteDatabase db, EntityEmpresa entity){
        if(db==null || entity==null)
                return false;

        ContentValues values = new ContentValues();
        values.put(COLUMN_NOM, entity.getNom());
        values.put(COLUMN_DIRECCIO, entity.getDireccio());
        values.put(COLUMN_CIUTAT, entity.getCiutat());
        values.put(COLUMN_TELEFON, entity.getTelefon());
        values.put(COLUMN_EMAIL, entity.getEmail());
        values.put(COLUMN_LOGO, entity.getLogo());
        values.put(COLUMN_LATITUD, entity.getLatitud());
        values.put(COLUMN_LONGITUD, entity.getLongitud());

        int rows = db.update(TABLE_NAME, values, COLUMN_ID+"="+entity.getId(), null);
        if(rows > 0){
        	return true;
        }
        else{
        	values.put(COLUMN_ID, entity.getId());
        	long row = db.insert(TABLE_NAME, null, values);
        	
        	if(row >= 0) return true;
        	else return false;
        }
	}	 
    
    public static boolean databaseDelete(SQLiteDatabase db, int id){
        if(db == null)
        	return false;
        
        int rows = db.delete(TABLE_NAME, COLUMN_ID+"="+id, null);
        if(rows > 0){
        	return true;
        }
        else{ 
        	return false;
        }
	}
    
    public static EntityEmpresa fromCursor(Cursor c) {
    	EntityEmpresa entity = new EntityEmpresa();

		if(c.getColumnIndex(COLUMN_ID)>=0) entity.setId( c.getInt(c.getColumnIndex(COLUMN_ID)) );
		if(c.getColumnIndex(COLUMN_NOM)>=0) entity.setNom( c.getString(c.getColumnIndex(COLUMN_NOM)) );
        if(c.getColumnIndex(COLUMN_DIRECCIO)>=0) entity.setDireccio( c.getString(c.getColumnIndex(COLUMN_DIRECCIO)) );
        if(c.getColumnIndex(COLUMN_CIUTAT)>=0) entity.setCiutat( c.getString(c.getColumnIndex(COLUMN_CIUTAT)) );
        if(c.getColumnIndex(COLUMN_TELEFON)>=0) entity.setTelefon( c.getString(c.getColumnIndex(COLUMN_TELEFON)) );
        if(c.getColumnIndex(COLUMN_EMAIL)>=0) entity.setEmail( c.getString(c.getColumnIndex(COLUMN_EMAIL)) );
        if(c.getColumnIndex(COLUMN_LOGO)>=0) entity.setLogo( c.getString(c.getColumnIndex(COLUMN_LOGO)) );
        if(c.getColumnIndex(COLUMN_LATITUD)>=0) entity.setLatitud( c.getDouble(c.getColumnIndex(COLUMN_LATITUD)) );
        if(c.getColumnIndex(COLUMN_LONGITUD)>=0) entity.setLongitud( c.getDouble(c.getColumnIndex(COLUMN_LONGITUD)) );

        return entity;
    }
}
