package cat.cordecor.crm.database;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import cat.cordecor.crm.http.RestResponse;
import cat.cordecor.crm.util.Log;
import android.content.ContentValues;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.os.Parcel;
import android.os.Parcelable;
import android.util.SparseArray;

public class EntityProducte implements Parcelable {

	// Database Table
    public static final String TABLE_NAME = "producte";
	
	// Database Columns
    public static final String COLUMN_ID = "id";
    public static final String COLUMN_NOM = "nom";
    public static final String COLUMN_CODI = "codi";
    public static final String COLUMN_DESCRIPCIO = "descripcio";
    public static final String COLUMN_PREU = "preu";
    public static final String COLUMN_FOTO = "foto";
	
	private int id;
    private String nom;
    private String codi;
    private String descripcio;
    private double preu;
    private String foto;

    public EntityProducte(){}

    public EntityProducte(Parcel in){
    	id = in.readInt();
        nom = in.readString();
        codi = in.readString();
        descripcio = in.readString();
        preu = in.readDouble();
        foto = in.readString();
	}	

    // Parcel
    public void writeToParcel(Parcel dest, int flags) {
    	dest.writeInt(id);
        dest.writeString(nom);
        dest.writeString(codi);
        dest.writeString(descripcio);
        dest.writeDouble(preu);
        dest.writeString(foto);
    }	
    
    public int describeContents() {
        return 0;
    }    
    
    public static final Creator<EntityProducte> CREATOR = new Creator<EntityProducte>() {
        public EntityProducte createFromParcel(Parcel in) {
            return new EntityProducte(in);
        }

        public EntityProducte[] newArray(int size) {
            return new EntityProducte[size];
        }
    };    

    // Getters & Setters
    public int getId(){ return id; }
    public String getNom(){ return nom; }
    public String getCodi(){ return codi; }
    public String getDescripcio(){ return descripcio; }
    public double getPreu(){ return preu; }
    public String getFoto(){ return foto; }

    public void setId(int value){ id = value; }
    public void setNom(String value){ nom = value; }
    public void setCodi(String value){ codi = value; }
    public void setDescripcio(String value){ descripcio = value; }
    public void setPreu(double value){ preu = value; }
    public void setFoto(String value){ foto = value; }

    public void setValues(EntityProducte entity){
    	if(entity != null){
			id = entity.getId();
			nom = entity.getNom();
			codi = entity.getCodi();
			descripcio = entity.getDescripcio();
			preu = entity.getPreu();
			foto = entity.getFoto();
    	}
    }
    
    public EntityProducte clone(){
    	EntityProducte entity = new EntityProducte();
    	entity.setValues(this);
    	return entity;
    }
    
    @Override
    public String toString() {
            return "EntityProducte [id=" + id
                            + ", nom=" + nom
                            + ", codi=" + codi
                            + ", descripcio=" + descripcio
                            + ", preu=" + preu
                            + ", foto=" + foto
                            + "]";
    }

    // JSON
    public static String toJSON(EntityProducte entity) {
        try {
            JSONObject json = new JSONObject();
            json.put(COLUMN_ID, entity.getId());
            json.put(COLUMN_NOM, entity.getNom());
            json.put(COLUMN_CODI, entity.getCodi());
            json.put(COLUMN_DESCRIPCIO, entity.getDescripcio());
            json.put(COLUMN_PREU, entity.getPreu());
            json.put(COLUMN_FOTO, entity.getFoto());

            return json.toString();
        }
        catch(JSONException e) {
            e.printStackTrace();
        }
        return null;
    }

    public static EntityProducte parseJSONEntity(JSONObject obj) throws JSONException{
    	EntityProducte entity = new EntityProducte();
        entity.setId(obj.getInt(COLUMN_ID));
        entity.setNom(obj.getString(COLUMN_NOM));
        entity.setCodi(obj.getString(COLUMN_CODI));
        entity.setDescripcio(obj.getString(COLUMN_DESCRIPCIO));
        entity.setPreu(obj.getDouble(COLUMN_PREU));
        entity.setFoto(obj.getString(COLUMN_FOTO));
        return entity;
    }
    
    public static SparseArray<EntityProducte> parseHttpGetAll(String jsonStr) {
    	Log.i("EntityProducte.parseHttpGetAll()"); 
    	SparseArray<EntityProducte> data = new SparseArray<EntityProducte>();
    	EntityProducte entity;
        try {
             JSONObject json = new JSONObject(jsonStr);
             JSONArray list = json.getJSONArray(RestResponse.TAG_RESULT);
             JSONObject obj;
             
             int size = list.length();
             for (int i=0; i<size; i++) {
            	 obj = list.getJSONObject(i);
                 entity = parseJSONEntity(obj);
                 data.put(entity.getId(), entity);
                 
                 DataManager.getInstance().modifyProducte(entity); // Modifica en DataManager
             }
        }
        catch (Exception e) {
        	data = null;
            e.printStackTrace();
        }
        return data;
    }
    
    public static SparseArray<EntityProducte> parseHttpGetSync(String jsonStr) {
    	Log.i("EntityProducte.parseHttpGetSync()"); 
    	SparseArray<EntityProducte> data = new SparseArray<EntityProducte>();
    	EntityProducte entity;
        try {
             JSONObject json = new JSONObject(jsonStr);
             json = json.getJSONObject(RestResponse.TAG_RESULT);
             JSONArray jsonDel = json.getJSONArray(RestResponse.TAG_RESULT_DELETED);
             JSONArray jsonMod = json.getJSONArray(RestResponse.TAG_RESULT_MODIFIED);
             JSONObject obj;
             int i,size,id;
             
             // Eliminats
             size = jsonDel.length();
             for (i=0; i<size; i++) {
            	 obj = jsonDel.getJSONObject(i);
            	 id = obj.getInt(COLUMN_ID);

                 DataManager.getInstance().deleteProducte(id); // Elimina en DataManager
             }
             
             // Modificats
             size = jsonMod.length();
             for (i=0; i<size; i++) {
            	 obj = jsonMod.getJSONObject(i);
                 entity = parseJSONEntity(obj);
                 data.put(entity.getId(), entity);
                 
                 DataManager.getInstance().modifyProducte(entity); // Modifica en DataManager
             }
        }
        catch (Exception e) {
        	data = null;
            e.printStackTrace();
        }
        return data;
    }
    
    // Database -------------------------------------------------
	public static String sqlCreateTable(){
                String sql = "CREATE TABLE "+TABLE_NAME+" ("
                + COLUMN_ID + " INTEGER NOT NULL, "
                + COLUMN_NOM + " TEXT, "
                + COLUMN_CODI + " TEXT, "
                + COLUMN_DESCRIPCIO + " TEXT, "
                + COLUMN_PREU + " REAL, "
                + COLUMN_FOTO + " TEXT, "
                + "PRIMARY KEY (" + COLUMN_ID + "));";
		return sql;
	}
	
	public static String sqlDropTable(){
                return "DROP TABLE IF EXISTS "+TABLE_NAME;
	}
	
    public static SparseArray<EntityProducte> databaseGetAll(SQLiteDatabase db){
        if(db == null)
        	return null;

        SparseArray<EntityProducte> data = new SparseArray<EntityProducte>();
		try{
			EntityProducte entity;
		    String sql = "SELECT * FROM "+TABLE_NAME;
		    Cursor cursor = db.rawQuery(sql, new String[] {});
		    
		    for(cursor.moveToFirst(); !cursor.isAfterLast(); cursor.moveToNext()){
		    	entity = fromCursor(cursor);
		    	data.append(entity.getId(), entity);
		    }
		}
		catch(Exception e){
			Log.e("EntityProducte.databaseGet(): "+e.toString());
		}
        return data;
	}
	
    public static boolean databaseUpdateInsert(SQLiteDatabase db, EntityProducte entity){
        if(db==null || entity==null)
                return false;

        ContentValues values = new ContentValues();
        values.put(COLUMN_NOM, entity.getNom());
        values.put(COLUMN_CODI, entity.getCodi());
        values.put(COLUMN_DESCRIPCIO, entity.getDescripcio());
        values.put(COLUMN_PREU, entity.getPreu());
        values.put(COLUMN_FOTO, entity.getFoto());

        int rows = db.update(TABLE_NAME, values, COLUMN_ID+"="+entity.getId(), null);
        if(rows > 0){
        	return true;
        }
        else{
        	values.put(COLUMN_ID, entity.getId());
        	long row = db.insert(TABLE_NAME, null, values);
        	
        	if(row >= 0) return true;
        	else return false;
        }
	}	
    
    public static boolean databaseDelete(SQLiteDatabase db, int id){
        if(db == null)
        	return false;
        
        int rows = db.delete(TABLE_NAME, COLUMN_ID+"="+id, null);
        if(rows > 0){
        	return true;
        }
        else{ 
        	return false;
        }
	}
    
    public static EntityProducte fromCursor(Cursor c) {
    	EntityProducte entity = new EntityProducte();

		if(c.getColumnIndex(COLUMN_ID)>=0) entity.setId( c.getInt(c.getColumnIndex(COLUMN_ID)) );
		if(c.getColumnIndex(COLUMN_NOM)>=0) entity.setNom( c.getString(c.getColumnIndex(COLUMN_NOM)) );
        if(c.getColumnIndex(COLUMN_CODI)>=0) entity.setCodi( c.getString(c.getColumnIndex(COLUMN_CODI)) );
        if(c.getColumnIndex(COLUMN_DESCRIPCIO)>=0) entity.setDescripcio( c.getString(c.getColumnIndex(COLUMN_DESCRIPCIO)) );
        if(c.getColumnIndex(COLUMN_PREU)>=0) entity.setPreu( c.getDouble(c.getColumnIndex(COLUMN_PREU)) );
        if(c.getColumnIndex(COLUMN_FOTO)>=0) entity.setFoto( c.getString(c.getColumnIndex(COLUMN_FOTO)) );

        return entity;
    }
}
