package cat.cordecor.crm.database;

import org.json.JSONException;
import org.json.JSONObject;

import cat.cordecor.crm.http.RestResponse;
import cat.cordecor.crm.util.Log;

import android.content.ContentValues;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.os.Parcel;
import android.os.Parcelable;

public class EntityUsuari implements Parcelable {

	// Database Table
    public static final String TABLE_NAME = "usuari";
	
	// Database Columns
    public static final String COLUMN_ID = "id";
    public static final String COLUMN_NOM = "nom";
    public static final String COLUMN_COGNOMS = "cognoms";
    public static final String COLUMN_TELEFON = "telefon";
    public static final String COLUMN_EMAIL = "email";
    public static final String COLUMN_CONTRASENYA = "contrasenya";
	
	private int id;
    private String nom;
    private String cognoms;
    private String telefon;
    private String email;
    private String contrasenya;

    public EntityUsuari(){}

    public EntityUsuari(Parcel in){
    	id = in.readInt();
        nom = in.readString();
        cognoms = in.readString();
        telefon = in.readString();
        email = in.readString();
        contrasenya = in.readString();
	}	

    // Parcel
    public void writeToParcel(Parcel dest, int flags) {
    	dest.writeInt(id);
        dest.writeString(nom);
        dest.writeString(cognoms);
        dest.writeString(telefon);
        dest.writeString(email);
        dest.writeString(contrasenya);
    }	
    
    public int describeContents() {
        return 0;
    }    
    
    public static final Creator<EntityUsuari> CREATOR = new Creator<EntityUsuari>() {
        public EntityUsuari createFromParcel(Parcel in) {
            return new EntityUsuari(in);
        }
  
        public EntityUsuari[] newArray(int size) {
            return new EntityUsuari[size];
        }
    };    

    // Getters & Setters
    public int getId(){ return id; }
    public String getNom(){ return nom; }
    public String getCognoms(){ return cognoms; }
    public String getTelefon(){ return telefon; }
    public String getEmail(){ return email; }
    public String getContrasenya(){ return contrasenya; }

    public void setId(int value){ id = value; }
    public void setNom(String value){ nom = value; }
    public void setCognoms(String value){ cognoms = value; }
    public void setTelefon(String value){ telefon = value; }
    public void setEmail(String value){ email = value; }
    public void setContrasenya(String value){ contrasenya = value; }

    public void setValues(EntityUsuari entity){
    	if(entity != null){
			id = entity.getId();
	        nom = entity.getNom();
	        cognoms = entity.getCognoms();
	        telefon = entity.getTelefon();
	        email = entity.getEmail();
	        contrasenya = entity.getContrasenya();
    	}
    }
    
    public EntityUsuari clone(){
    	EntityUsuari entity = new EntityUsuari();
    	entity.setValues(this);
    	return entity;
    }
    
    @Override
    public String toString() {
            return "EntityUsuari [id=" + id
                            + ", nom=" + nom
                            + ", cognoms=" + cognoms
                            + ", telefon=" + telefon
                            + ", email=" + email
                            + ", contrasenya=" + contrasenya
                            + "]";
    }

    // JSON
    public static String toJSON(EntityUsuari entity) {
        try {
            JSONObject json = new JSONObject();
            json.put(COLUMN_ID, entity.getId());
            json.put(COLUMN_NOM, entity.getNom());
            json.put(COLUMN_COGNOMS, entity.getCognoms());
            json.put(COLUMN_TELEFON, entity.getTelefon());
            json.put(COLUMN_EMAIL, entity.getEmail());
            json.put(COLUMN_CONTRASENYA, entity.getContrasenya());

            return json.toString();
        }
        catch(JSONException e) {
            e.printStackTrace();
        }
        return null;
    }

    public static EntityUsuari parseJSON(String jsonStr) {
        EntityUsuari entity;
        try {
             JSONObject json = new JSONObject(jsonStr);
             json = json.getJSONObject(RestResponse.TAG_RESULT);

             entity = new EntityUsuari();
             entity.setId(json.getInt(COLUMN_ID));
             entity.setNom(json.getString(COLUMN_NOM));
             entity.setCognoms(json.getString(COLUMN_COGNOMS));
             entity.setTelefon(json.getString(COLUMN_TELEFON));
             entity.setEmail(json.getString(COLUMN_EMAIL));
             entity.setContrasenya(json.getString(COLUMN_CONTRASENYA));
        }
        catch (Exception e) {
           entity = null;
           e.printStackTrace();
        }
        return entity;
    }

    // Database -------------------------------------------------
	public static String sqlCreateTable(){
                String sql = "CREATE TABLE "+TABLE_NAME+" ("
                + COLUMN_ID + " INTEGER NOT NULL, "
                + COLUMN_NOM + " TEXT, "
                + COLUMN_COGNOMS + " TEXT, "
                + COLUMN_TELEFON + " TEXT, "
                + COLUMN_EMAIL + " TEXT, "
                + COLUMN_CONTRASENYA + " TEXT, "
                + "PRIMARY KEY (" + COLUMN_ID + "));";
		return sql;
	}
	
	public static String sqlDropTable(){
                return "DROP TABLE IF EXISTS "+TABLE_NAME;
	}
	
    public static EntityUsuari databaseGet(SQLiteDatabase db, int id){
        if(db == null)
        	return null;

        EntityUsuari entity = null;
		try{
		    String sql = "SELECT * FROM "+TABLE_NAME+" WHERE "+COLUMN_ID+"="+id;
		    Cursor cursor = db.rawQuery(sql, new String[] {});
		    
		    int count = 0;
		    for(cursor.moveToFirst(); !cursor.isAfterLast() && count==0; cursor.moveToNext()){
		    	entity = fromCursor(cursor);
		    	count++;
		    }
		}
		catch(Exception e){
			Log.e("EntityUsuari.databaseGet(): "+e.toString());
		}
        return entity;
	}	
	
    public static boolean databaseUpdateInsert(SQLiteDatabase db, EntityUsuari entity){
        if(db==null || entity==null)
                return false;

        ContentValues values = new ContentValues();
        values.put(COLUMN_NOM, entity.getNom());
        values.put(COLUMN_COGNOMS, entity.getCognoms());
        values.put(COLUMN_TELEFON, entity.getTelefon());
        values.put(COLUMN_EMAIL, entity.getEmail());
        values.put(COLUMN_CONTRASENYA, entity.getContrasenya());

        int rows = db.update(TABLE_NAME, values, COLUMN_ID+"="+entity.getId(), null);
        if(rows > 0){
        	return true;
        }
        else{
        	values.put(COLUMN_ID, entity.getId());
        	long row = db.insert(TABLE_NAME, null, values);
        	
        	if(row >= 0) return true;
        	else return false;
        }
	}		
    
    public static EntityUsuari fromCursor(Cursor c) {
    	EntityUsuari entity = new EntityUsuari();

		if(c.getColumnIndex(COLUMN_ID)>=0) entity.setId( c.getInt(c.getColumnIndex(COLUMN_ID)) );
		if(c.getColumnIndex(COLUMN_NOM)>=0) entity.setNom( c.getString(c.getColumnIndex(COLUMN_NOM)) );
        if(c.getColumnIndex(COLUMN_COGNOMS)>=0) entity.setCognoms( c.getString(c.getColumnIndex(COLUMN_COGNOMS)) );
        if(c.getColumnIndex(COLUMN_TELEFON)>=0) entity.setTelefon( c.getString(c.getColumnIndex(COLUMN_TELEFON)) );
        if(c.getColumnIndex(COLUMN_EMAIL)>=0) entity.setEmail( c.getString(c.getColumnIndex(COLUMN_EMAIL)) );
        if(c.getColumnIndex(COLUMN_CONTRASENYA)>=0) entity.setContrasenya( c.getString(c.getColumnIndex(COLUMN_CONTRASENYA)) );

        return entity;
    }
}
