package cat.cordecor.crm.http;

import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.URL;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.conn.params.ConnManagerParams;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.params.HttpConnectionParams;
import org.apache.http.params.HttpParams;

import android.content.ContextWrapper;
import android.graphics.Bitmap;
import cat.cordecor.crm.App;
import cat.cordecor.crm.util.Log;

public class HttpImage {
	public static final int HTTP_TIMEOUT = 15000;	
	public static final String LOCAL_FOLDER = "images";
	
	private static final DefaultHttpClient httpClient; 
    static {
    	httpClient = new DefaultHttpClient();
    	
		HttpParams params = httpClient.getParams();
		HttpConnectionParams.setConnectionTimeout(params, HTTP_TIMEOUT);
		HttpConnectionParams.setSoTimeout(params, HTTP_TIMEOUT);
		ConnManagerParams.setTimeout(params, HTTP_TIMEOUT); 
    }
    
    public static File download(String url, String localFile){ 
    	Log.i("HttpImage.download() /url: "+url);
    	File file = null;
    	DefaultHttpClient client = null;
    	HttpGet getRequest = null;
    	long startTime;
        try {
            client = httpClient;
            getRequest = new HttpGet(url);
            startTime = System.currentTimeMillis();
            
            HttpResponse response = client.execute(getRequest);
            final int statusCode = response.getStatusLine().getStatusCode();
            if (statusCode != HttpStatus.SC_OK) { 
            	Log.i("HttpImage.download() /connection error: "+url);
                return null;
            }
            
            final HttpEntity entity = response.getEntity();
            if (entity != null) {
                InputStream inputStream = null;
                try {
                    inputStream = entity.getContent();                      
                    try {
                      	ContextWrapper cw = new ContextWrapper(App.getAppContext());
                      	File dirImages = cw.getFilesDir();
                        file = new File(dirImages, localFile);
                        final OutputStream output = new FileOutputStream(file);
                        try {
                            try {
                                final byte[] buffer = new byte[1024];
                                int read;

                                while ((read = inputStream.read(buffer)) != -1)
                                    output.write(buffer, 0, read);

                                output.flush();
                            } finally {
                                if(output != null) output.close();
                            }
                        } catch (Exception e) {
                            e.printStackTrace();
                        }
                    } finally {
                    	if(inputStream != null) inputStream.close();
                    }                                         
                    return file;
                } 
                finally {
                    if (inputStream != null) {
                        inputStream.close();  
                    }
                    if(entity != null) entity.consumeContent();
                    
                    Log.i("HttpImage.download() /url:"+url+" /miliseconds:"
                            + (System.currentTimeMillis() - startTime));                           
                }
            }
        } 
        catch (Exception e) {
        	Log.e("HttpImage.download() /error: "+e.toString());
            if(getRequest != null) getRequest.abort();
        } 
        finally {
            //if (client != null) {
            //    client.close();
            //}
        }
        Log.e("HttpImage.download() /end");
        return null;
    }     
 
    public static String upload(String url, String fileName){
    	Log.i("HttpImage.upload()");
    	
    	String response = null;
    	FileInputStream fileInputStream;
        String lineEnd = "\r\n";
        String twoHyphens = "--";
        String boundary = "*****";
        try{
        	URL connectURL = new URL(url);
	    	File dirImages = App.getAppContext().getFilesDir();
	        File file = new File(dirImages, fileName);
        	fileInputStream = new FileInputStream(file);
        	
            HttpURLConnection conn = (HttpURLConnection) connectURL.openConnection();
            conn.setDoInput(true);
            conn.setDoOutput(true);
            conn.setUseCaches(false);
            conn.setRequestMethod("POST");
            conn.setRequestProperty("Connection", "Keep-Alive");
            conn.setRequestProperty("Content-Type", "multipart/form-data;boundary="+boundary);
 
            DataOutputStream dos = new DataOutputStream( conn.getOutputStream() );
 
            dos.writeBytes(twoHyphens + boundary + lineEnd);
            dos.writeBytes("Content-Disposition: form-data; name=\"uploadedfile\"; filename=\"" + fileName +"\"" + lineEnd);
            dos.writeBytes(lineEnd);
 
            Log.i("/Headers are written");
            int bytesAvailable = fileInputStream.available();
            int maxBufferSize = 1024;
            int bufferSize = Math.min(bytesAvailable, maxBufferSize);
            byte[] buffer = new byte[bufferSize];
            int bytesRead = fileInputStream.read(buffer, 0, bufferSize);
 
            while (bytesRead > 0){
	            dos.write(buffer, 0, bufferSize);
	            bytesAvailable = fileInputStream.available();
	            bufferSize = Math.min(bytesAvailable, maxBufferSize);
	            bytesRead = fileInputStream.read(buffer, 0, bufferSize);
            }
 
            dos.writeBytes(lineEnd);
            dos.writeBytes(twoHyphens + boundary + twoHyphens + lineEnd);
 
            Log.i("/File is written");
            fileInputStream.close();
            dos.flush();
 
            InputStream is = conn.getInputStream();
            int ch;
            StringBuffer b = new StringBuffer();
            while( ( ch = is.read() ) != -1 ){
            	b.append( (char)ch );
            }
            response = b.toString();
            dos.close();
 
            Log.i("/response: " + response);
        }
        catch (Exception e){
        	response = null;
        	Log.e("/error: "+ e.toString());
        }
        return response;
    }
}
