package cat.cordecor.crm.http;

import android.util.SparseArray;
import cat.cordecor.crm.database.DataManager;
import cat.cordecor.crm.database.EntityComanda;
import cat.cordecor.crm.database.EntityContacte;
import cat.cordecor.crm.database.EntityEmpresa;
import cat.cordecor.crm.database.EntityProducte;
import cat.cordecor.crm.database.EntityUsuari;
import cat.cordecor.crm.util.Log;

public class HttpManager
{
	static public final String REST_URL = "http://www.javierms.com/cordecor/index.php/rest/";
	static public final String DOWNLOAD_URL = "http://www.javierms.com/cordecor/application/controllers/images/";
	static public final String UPLOAD_URL = "http://www.javierms.com/cordecor/application/controllers/upload.php";
	
	// Usuari ------------------------------------
	public static EntityUsuari login(String userEmail, String userPassword) {
		EntityUsuari entity = null;

		String url = REST_URL + "usuari";
		String responseStr = HttpMethods.doGet(url, userEmail, userPassword); 
		Log.i("HttpManager.login() /response: "+responseStr); 
		
		RestResponse response = RestResponse.parseJSON(responseStr);
 		if(response!=null && !response.hasError()){ 
 			entity = EntityUsuari.parseJSON(responseStr);
 			if(entity != null){
 				Log.i("/entity.toString(): "+entity.toString());
 				DataManager.getInstance().modifyUsuari(entity);
 			}
 		}
 		if(entity == null) Log.i("/error");
 		
     	return entity;
	}
	
	public static EntityUsuari getUsuari() {
		EntityUsuari entity = null;

		String url = REST_URL + "usuari";
		String responseStr = httpGet(url);
		Log.i("HttpManager.getUsuari() /response: "+responseStr); 
		
		RestResponse response = RestResponse.parseJSON(responseStr);
 		if(response!=null && !response.hasError()){ 
 			entity = EntityUsuari.parseJSON(responseStr);
 			if(entity != null){
 				Log.i("/entity.toString(): "+entity.toString());
 				DataManager.getInstance().modifyUsuari(entity);
 			}
 		}
 		if(entity == null) Log.i("/error");
		
     	return entity;
	}
	
	public static boolean updateUsuari(EntityUsuari entity) {
		if(entity == null)
			return false;
		
		Log.i("HttpManager.updateUsuari()"); 
		String url = REST_URL + "usuari";
		String data = EntityUsuari.toJSON(entity);
		Log.i("/data: "+data);
		String responseStr = httpPost(url, data);
		Log.i("/response: "+responseStr); 
		
		RestResponse response = RestResponse.parseJSON(responseStr);
 		if(response!=null && !response.hasError()){ 
 			Log.i("/ok");
 			DataManager.getInstance().modifyUsuari(entity);
 			return true;
 		}
 		else{
 			Log.i("/error");
 			return false;
 		}
	}
	
	// Empresa ------------------------------------
	public static SparseArray<EntityEmpresa> getEmpreses() {
		Log.i("HttpManager.getEmpreses()");
		EntityUsuari user = DataManager.getInstance().getUsuari();
		SparseArray<EntityEmpresa> data = null;
		
		if(user != null){
			SparseArray<EntityEmpresa> recover = DataManager.getInstance().getEmpreses();
			boolean syncMode = (recover!=null && recover.size()>0) ? true : false;
			
			String url = (syncMode) ? REST_URL + "empresesSync" : REST_URL + "empreses"; 			
			String responseStr = httpGet(url);
			Log.i("/response: "+responseStr); 
			
			RestResponse response = RestResponse.parseJSON(responseStr);
	 		if(response!=null && !response.hasError()){ 
	 			if(response.isOkEmpty()){
	 				data = new SparseArray<EntityEmpresa>();
	 			}
	 			else{
	 	 			if(syncMode){
		 				data = EntityEmpresa.parseHttpGetSync(responseStr);
		 			}
		 			else{
		 				data = EntityEmpresa.parseHttpGetAll(responseStr);
		 			}
	 			}
	 		}
		}
 		if(data == null) Log.i("/error");
 		
     	return data;
	}
	
	public static boolean insertEmpresa(EntityEmpresa entity) {
		Log.i("HttpManager.insertEmpresa()"); 
		
		if(entity == null)
			return false;
		
		String url = REST_URL + "empresa";
		String data = EntityEmpresa.toJSON(entity);
		Log.i("/data: "+data);
		String responseStr = httpPut(url, data);
		Log.i("/response: "+responseStr); 
		
		RestResponse response = RestResponse.parseJSON(responseStr);
 		if(response!=null && !response.hasError() && response.getNewID()>0){ 
 			Log.i("/ok");
 			entity.setId(response.getNewID());
 			DataManager.getInstance().modifyEmpresa(entity);
 			return true;
 		}
 		else{
 			Log.i("/error");
 			return false;
 		}
	}
	
	public static boolean updateEmpresa(EntityEmpresa entity, boolean uploadPhoto) {
		Log.i("HttpManager.updateEmpresa()"); 
		
		if(entity == null)
			return false;
		
		String responseStr;
		RestResponse response;
		
		// Primer pugem la foto si cal
		if(uploadPhoto){
			responseStr = HttpImage.upload(UPLOAD_URL, entity.getLogo());
			response = RestResponse.parseJSON(responseStr);
			if(response==null || response.hasError()){ 
	 			Log.i("/error"); 
	 			return false;
	 		}
		}
		
		// Ara fem la crida REST
		String url = REST_URL + "empresa";
		String data = EntityEmpresa.toJSON(entity);
		Log.i("/data: "+data);
		responseStr = httpPost(url, data);
		Log.i("/response: "+responseStr); 
		
		response = RestResponse.parseJSON(responseStr);
 		if(response!=null && !response.hasError()){ 
 			Log.i("/ok");
 			DataManager.getInstance().modifyEmpresa(entity);
 			return true;
 		}
 		else{
 			Log.i("/error");
 			return false;
 		}
	}
	
	// Contacte ------------------------------------
	public static SparseArray<EntityContacte> getContactes() {
		Log.i("HttpManager.getContactes()");
		EntityUsuari user = DataManager.getInstance().getUsuari();
		SparseArray<EntityContacte> data = null;
		
		if(user != null){
			SparseArray<EntityContacte> recover = DataManager.getInstance().getContactes();
			boolean syncMode = (recover!=null && recover.size()>0) ? true : false;
			
			String url = (syncMode) ? REST_URL + "contactesSync" : REST_URL + "contactes"; 
			String responseStr = httpGet(url);
			Log.i("/response: "+responseStr); 
			
			RestResponse response = RestResponse.parseJSON(responseStr);
	 		if(response!=null && !response.hasError()){ 
	 			if(response.isOkEmpty()){
	 				data = new SparseArray<EntityContacte>();
	 			}
	 			else{
	 	 			if(syncMode){
		 				data = EntityContacte.parseHttpGetSync(responseStr);
		 			}
		 			else{
		 				data = EntityContacte.parseHttpGetAll(responseStr);
		 			}
	 			}
	 		}
		}
 		if(data == null) Log.i("/error");
 		
     	return data;
	}
	
	public static boolean insertContacte(EntityContacte entity) {
		Log.i("HttpManager.insertContacte()"); 
		
		if(entity == null)
			return false;
		
		String url = REST_URL + "contacte";
		String data = EntityContacte.toJSON(entity);
		Log.i("/data: "+data);
		String responseStr = httpPut(url, data);
		Log.i("/response: "+responseStr); 
		
		RestResponse response = RestResponse.parseJSON(responseStr);
 		if(response!=null && !response.hasError() && response.getNewID()>0){ 
 			Log.i("/ok");
 			entity.setId(response.getNewID());
 			DataManager.getInstance().modifyContacte(entity);
 			return true;
 		}
 		else{
 			Log.i("/error");
 			return false;
 		}
	}
	
	public static boolean updateContacte(EntityContacte entity, boolean uploadPhoto) {
		Log.i("HttpManager.updateContacte()"); 
		
		if(entity == null)
			return false;
		
		String responseStr;
		RestResponse response;
		
		// Primer pugem la foto si cal
		if(uploadPhoto){
			responseStr = HttpImage.upload(UPLOAD_URL, entity.getFoto());
			response = RestResponse.parseJSON(responseStr);
			if(response==null || response.hasError()){ 
	 			Log.i("/error"); 
	 			return false;
	 		}
		}
		
		// Ara fem la crida REST
		String url = REST_URL + "contacte";
		String data = EntityContacte.toJSON(entity);
		Log.i("/data: "+data);
		responseStr = httpPost(url, data);
		Log.i("/response: "+responseStr); 
		
		response = RestResponse.parseJSON(responseStr);
 		if(response!=null && !response.hasError()){ 
 			Log.i("/ok");
 			DataManager.getInstance().modifyContacte(entity);
 			return true;
 		}
 		else{
 			Log.i("/error");
 			return false;
 		}
	}
	
	public static boolean deleteContacte(int id) {
		Log.i("HttpManager.deleteContacte()"); 
		
		String url = REST_URL + "contacte/id/"+id;
		String responseStr = httpDelete(url);
		Log.i("/response: "+responseStr); 
		
		RestResponse response = RestResponse.parseJSON(responseStr);
 		if(response!=null && !response.hasError()){ 
 			Log.i("/ok");
 			DataManager.getInstance().deleteContacte(id);
 			return true;
 		}
 		else{
 			Log.i("/error");
 			return false;
 		}
	}
	
	// Producte ------------------------------------
	public static SparseArray<EntityProducte> getProductes() {
		Log.i("HttpManager.getProductes()");
		EntityUsuari user = DataManager.getInstance().getUsuari();
		SparseArray<EntityProducte> data = null;
		
		if(user != null){
			SparseArray<EntityProducte> recover = DataManager.getInstance().getProductes();
			boolean syncMode = (recover!=null && recover.size()>0) ? true : false;
			
			String url = (syncMode) ? REST_URL + "productesSync" : REST_URL + "productes"; 	
			String responseStr = httpGet(url);
			Log.i("/response: "+responseStr); 
			
			RestResponse response = RestResponse.parseJSON(responseStr);
	 		if(response!=null && !response.hasError()){ 
	 			if(response.isOkEmpty()){
	 				data = new SparseArray<EntityProducte>();
	 			}
	 			else{
	 	 			if(syncMode){
		 				data = EntityProducte.parseHttpGetSync(responseStr);
		 			}
		 			else{
		 				data = EntityProducte.parseHttpGetAll(responseStr);
		 			}
	 			}
	 		}
		}
 		if(data == null) Log.i("/error");
 		
     	return data;
	}
	
	// Comanda ------------------------------------
	public static SparseArray<EntityComanda> getComandes() {
		Log.i("HttpManager.getComandes()");
		EntityUsuari user = DataManager.getInstance().getUsuari();
		SparseArray<EntityComanda> data = null;
		
		if(user != null){
			SparseArray<EntityComanda> recover = DataManager.getInstance().getComandes();
			boolean syncMode = (recover!=null && recover.size()>0) ? true : false;
			
			String url = (syncMode) ? REST_URL + "comandesSync" : REST_URL + "comandes"; 
			String responseStr = httpGet(url);
			Log.i("/response: "+responseStr); 
			
			RestResponse response = RestResponse.parseJSON(responseStr);
	 		if(response!=null && !response.hasError()){ 
	 			if(response.isOkEmpty()){
	 				data = new SparseArray<EntityComanda>();
	 			}
	 			else{
	 	 			if(syncMode){
		 				data = EntityComanda.parseHttpGetSync(responseStr);
		 			}
		 			else{
		 				data = EntityComanda.parseHttpGetAll(responseStr);
		 			}
	 			}
	 		}
		}
 		if(data == null) Log.i("/error");
 		
     	return data;
	}
	
	public static boolean insertComanda(EntityComanda entity) {
		Log.i("HttpManager.insertComanda()"); 
		
		if(entity == null)
			return false;
		
		String url = REST_URL + "comanda";
		String data = EntityComanda.toJSON(entity);
		Log.i("/data: "+data);
		String responseStr = httpPut(url, data);
		Log.i("/response: "+responseStr); 
		
		RestResponse response = RestResponse.parseJSON(responseStr);
 		if(response!=null && !response.hasError() && response.getNewID()>0){ 
 			Log.i("/ok");
 			entity.setId(response.getNewID());
 			DataManager.getInstance().modifyComanda(entity);
 			return true;
 		}
 		else{
 			Log.i("/error");
 			return false;
 		}
	}
	
	// ====================================================
	
	public static String httpGet(String url) {
		EntityUsuari user = DataManager.getInstance().getUsuari();
		String email = (user != null) ? user.getEmail() : "";
		String pwd = (user != null) ? user.getContrasenya() : "";
		
		return HttpMethods.doGet(url, email, pwd); 
	}
	
	public static String httpPost(String url, String content) {
		EntityUsuari user = DataManager.getInstance().getUsuari();
		String email = (user != null) ? user.getEmail() : "";
		String pwd = (user != null) ? user.getContrasenya() : "";
		
		return HttpMethods.doPost(url, content, email, pwd); 
	}	
	
	public static String httpPut(String url, String content) {
		EntityUsuari user = DataManager.getInstance().getUsuari();
		String email = (user != null) ? user.getEmail() : "";
		String pwd = (user != null) ? user.getContrasenya() : "";
		
		return HttpMethods.doPut(url, content, email, pwd); 
	}		
	
	public static String httpDelete(String url) {
		EntityUsuari user = DataManager.getInstance().getUsuari();
		String email = (user != null) ? user.getEmail() : "";
		String pwd = (user != null) ? user.getContrasenya() : "";
		
		return HttpMethods.doDelete(url, email, pwd); 
	}	
	
	/*private static String idToJSON(int id){
		String jsonStr = null;
	    try {
	    	JSONObject json = new JSONObject();
	        json.put("id", id);
	        
	        return json.toString();
	    }
	    catch(JSONException e) {
	    	jsonStr = null;
	        e.printStackTrace();
	    }
	    return jsonStr;
	}*/
}