package cat.cordecor.crm.http;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.HttpVersion;
import org.apache.http.client.methods.HttpDelete;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpPut;
import org.apache.http.client.methods.HttpUriRequest;
import org.apache.http.client.params.HttpClientParams;
import org.apache.http.conn.ClientConnectionManager;
import org.apache.http.conn.params.ConnManagerParams;
import org.apache.http.conn.scheme.PlainSocketFactory;
import org.apache.http.conn.scheme.Scheme;
import org.apache.http.conn.scheme.SchemeRegistry;
import org.apache.http.conn.ssl.SSLSocketFactory;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.impl.conn.tsccm.ThreadSafeClientConnManager;
import org.apache.http.params.BasicHttpParams;
import org.apache.http.params.HttpConnectionParams;
import org.apache.http.params.HttpParams;
import org.apache.http.params.HttpProtocolParams;
import org.apache.http.protocol.HTTP;
import org.apache.http.util.EntityUtils;

import android.util.Base64;
import cat.cordecor.crm.util.Log;

public class HttpMethods {
	static final public String JSON = "json";
	static final public String UTF_8 = HTTP.UTF_8;
	static final public String ISO_8859_1 = HTTP.ISO_8859_1;
	
	static final public int CONNECTION_ATTEMPTS = 2;
	static final public int HTTP_TIMEOUT = 15 * 1000; 
	
	private static final DefaultHttpClient httpClient; 
    static {
        final HttpParams params = new BasicHttpParams();
        HttpProtocolParams.setVersion(params, HttpVersion.HTTP_1_1);

        ConnManagerParams.setTimeout(params, HTTP_TIMEOUT);
        HttpConnectionParams.setConnectionTimeout(params, HTTP_TIMEOUT);
        HttpConnectionParams.setSoTimeout(params, HTTP_TIMEOUT);
        HttpConnectionParams.setStaleCheckingEnabled(params, false);
        HttpConnectionParams.setSocketBufferSize(params, 8192);

        HttpClientParams.setRedirecting(params, false);

        SchemeRegistry schemeRegistry = new SchemeRegistry();
        schemeRegistry.register(new Scheme("http", PlainSocketFactory.getSocketFactory(), 80));
        schemeRegistry.register(new Scheme("https", SSLSocketFactory.getSocketFactory(), 443));

        ClientConnectionManager manager = new ThreadSafeClientConnManager(params, schemeRegistry);
        httpClient = new DefaultHttpClient(manager, params);
    }
    
	// =================================================
	
	/**
	 * HTTP (get)
	 * @return String response
	 */
    public static String doGet(String url, String username, String password){
    	Log.i("HttpMethods.doGet() /url: "+url);
    	
    	HttpGet request = new HttpGet(url);

    	request.setHeader(HTTP.CONTENT_TYPE, "application/" + JSON); 
        
        return doRequest(request, username, password);
    }	 

    /**
     * HTTP (delete)
     * @return String response
     */
    public static String doDelete(String url, String username, String password){
    	Log.i("HttpMethods.doDelete() /url: "+url);
    	
    	HttpDelete request = new HttpDelete(url);

        request.setHeader(HTTP.CONTENT_TYPE, "application/" + JSON); 
        
        return doRequest(request, username, password);
    }    
    
    /**
     * HTTP (post)
     * @return String response
     */
    public static String doPost(String url, String content, String username, String password){
    	Log.i("HttpMethods.doPost() /url: "+url);
    	
    	HttpPost request = new HttpPost(url);
    	StringEntity s = getStringEntity(request, content);
    	if(s != null){
    		request.setEntity(s);
    		return doRequest(request, username, password);
    	}
    	else{
    		request.abort();
    		return null;
    	} 
    }
    
    /**
     * HTTP (put)
     * @return String response
     */
    public static String doPut(String url, String content, String username, String password){
    	Log.i("HttpMethods.doPut() /url: "+url);
    	
    	HttpPut request = new HttpPut(url);
    	StringEntity s = getStringEntity(request, content);
    	if(s != null){
    		request.setEntity(s);
    		return doRequest(request, username, password);
    	}
    	else{
    		request.abort();
    		return null;
    	}
    }
    
    // =================================================
    
    private static String doRequest(HttpUriRequest request, String username, String password){
        try {
        	request.setHeader("Authorization", "Basic " + Base64.encodeToString((username+":"+password).getBytes(), Base64.NO_WRAP));
        	
            HttpEntity entity = executeRequest(httpClient, request, 0);
            if(entity != null) {
                return EntityUtils.toString(entity, UTF_8);
            }
        } 
        catch(Exception e) {
        	request.abort();
            Log.e("HttpMethods.doPostRequest() /Exception: "+e.toString());
        }        
        return null;    	
    }  
     
    public static StringEntity getStringEntity(HttpUriRequest request, String content){
        try{
        	byte[] bute = null;
            bute = content.getBytes();
            try {
            	content = new String(bute, ISO_8859_1);
            } 
            catch (Exception e) {
                e.printStackTrace();
            }
            
            StringEntity s = new StringEntity(content);
            s.setContentEncoding(UTF_8);
            s.setContentType("application/" + JSON); 
            
            request.setHeader(HTTP.CONTENT_TYPE, "application/" + JSON + "; charset=" + UTF_8);
            
            return s;
        }
        catch(Exception e) {
            Log.e("HttpMethods.getStringEntity() /Exception: "+e.toString());
            return null;
        }
    }
    
    /**
     * Si falla la connexi far varis intents
     * Nombre d'intents declarats en la variable CONNECTION_ATTEMPTS
     * @param client
     * @param request
     * @return
     */
    public static HttpEntity executeRequest(DefaultHttpClient client, HttpUriRequest request){
    	return executeRequest(client, request, 0);
    }    
    
    private static HttpEntity executeRequest(DefaultHttpClient client, HttpUriRequest request, int currentAttempt){
    	HttpResponse response;
    	int statusCode;
    	try{
    		response = client.execute(request);
    		statusCode = response.getStatusLine().getStatusCode();
    	}
    	catch(Exception e){
    		statusCode = -1;
    		response = null;
    	}
    	
        if(response==null || statusCode!=HttpStatus.SC_OK) { // Si no s OK
        	Log.i("HttpMethods.executeRequest() /statusCode:"+statusCode+" /currentAttempt:"+currentAttempt);        	
        	currentAttempt++;
        	if(currentAttempt < CONNECTION_ATTEMPTS){ 
        		return executeRequest(client, request, currentAttempt);
        	}
        	else{ 
        		Log.e("HttpMethods.executeRequest(): nombre d'intents de connexi superats");  
        		return null;
        	}
        } 
        else{
        	return response.getEntity();
        }
    }        
}