package cat.cordecor.crm.util;

import java.io.File;
import java.io.FileOutputStream;
import java.text.SimpleDateFormat;
import java.util.Calendar;

import cat.cordecor.crm.App;
import cat.cordecor.crm.R;
import cat.cordecor.crm.database.DataManager;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.ContextWrapper;
import android.content.DialogInterface;
import android.graphics.Bitmap;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.AsyncTask;
import android.view.View;
import android.widget.ImageButton;
import android.widget.TextView;

public class Utils 
{		
	public static void configTopActivity(final Activity activity, String title){
		TextView topTitle = (TextView) activity.findViewById(R.id.topTitle);
		topTitle.setText(title);
		
		ImageButton btnBack = (ImageButton) activity.findViewById(R.id.btnBack);
		btnBack.setVisibility(ImageButton.VISIBLE);
		btnBack.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				activity.onBackPressed();
			}
		});
	}
	
	public static boolean isNetworkConnected(Context ctx) {
		Log.i("Utils.isNetworkConnected()"); 
	    boolean connected = false;
	    ConnectivityManager manager = (ConnectivityManager) ctx.getSystemService(Context.CONNECTIVITY_SERVICE);
	    
	    NetworkInfo[] redes = manager.getAllNetworkInfo();
	    for (int i = 0; i < 2; i++) {
	        if (redes[i].getState() == NetworkInfo.State.CONNECTED) {
	        	connected = true;
	        }
	    }
	    return connected;
	}
	
	public static void exit(){
		Log.i("Utils.exit()"); 
		new AsyncTask<Void, Void, Void>(){
    		@Override
    		protected Void doInBackground (Void... params){
    			DataManager.getInstance().deleteLoggedUser();
    			return null;
    		}
    		
    		@Override
    	    protected void onPostExecute(Void params) {
    			System.exit(0);	
    	    }
		}.execute();		
	}
	
	public static void exitDialog(Context context, String message) {
		Log.i("Utils.exitDialog()"); 
		try{
			AlertDialog.Builder builder = new AlertDialog.Builder(context);
			builder.setCancelable(false)
					.setMessage(message)
					.setNegativeButton(context.getString(R.string.btn_sortir), new DialogInterface.OnClickListener() {
						public void onClick(DialogInterface dialog, int id) {
							exit();
						}
					});
			AlertDialog dialog = builder.create();
			dialog.setCancelable(false);
			dialog.show();
		}
		catch(Exception e){
			Log.e("/error:"+e.toString()); 
		}
	}
	
	public static void dialogValorsNoModificats(Context context) {
		Log.i("Utils.dialogValorsNoModificats()"); 
		try{
			AlertDialog.Builder builder = new AlertDialog.Builder(context);
			builder.setCancelable(false)
					.setMessage(context.getString(R.string.valorsNoModificats))
					.setPositiveButton(context.getString(R.string.btn_ok), null);
			AlertDialog dialog = builder.create();
			dialog.setCancelable(false);
			dialog.show();
		}
		catch(Exception e){
			Log.e("/error:"+e.toString()); 
		}
	}
	
	public static void dialogCampsBuits(Context context, String camps) {
		Log.i("Utils.dialogCampsBuits()"); 
		try{
			String msg = context.getString(R.string.omplirCamps)+" "+camps;
			
			AlertDialog.Builder builder = new AlertDialog.Builder(context);
			builder.setCancelable(false)
					.setMessage(msg)
					.setPositiveButton(context.getString(R.string.btn_ok), null);
			AlertDialog dialog = builder.create();
			dialog.setCancelable(false);
			dialog.show();
		}
		catch(Exception e){
			Log.e("/error:"+e.toString()); 
		}
	}
	
	public static void dialogHttpRestError(Context context) {
		Log.i("Utils.dialogHttpRestError()"); 
		try{
			AlertDialog.Builder builder = new AlertDialog.Builder(context);
			builder.setCancelable(false)
					.setMessage(context.getString(R.string.httpRestError))
					.setNegativeButton(context.getString(R.string.btn_tancar), null);
			AlertDialog dialog = builder.create();
			dialog.setCancelable(false);
			dialog.show();
		}
		catch(Exception e){
			Log.e("/error:"+e.toString()); 
		}
	}
	
	public static String getAbsolutePath(String localFile){
    	String path = "";
    	try{
	    	ContextWrapper cw = new ContextWrapper(App.getAppContext());
	    	File dirImages = cw.getFilesDir();
	        File file = new File(dirImages, localFile);
    	    if(file.exists()){
    	        path = file.getAbsolutePath().toString();
    	    }
    	}
    	catch(Exception e){
    		Log.e("Utils.getAbsolutePath(): "+e.toString());
    		path = "";
    	}
        return path;
    }
    
	public static String bitmapToLocalFile(Bitmap image, String fileName){
		Log.i("Utils.bitmapToLocalFile()");
		String path = null;
		try{
			ContextWrapper cw = new ContextWrapper(App.getAppContext());
			File dirImages = cw.getFilesDir();
			File myPath = new File(dirImages, fileName);
			FileOutputStream fos = new FileOutputStream(myPath);
			image.compress(Bitmap.CompressFormat.JPEG, 80, fos);
			fos.flush();
			fos.close();
			path = myPath.getAbsolutePath();
		}
		catch (Exception e){
			path = null;
			Log.e("Utils.bitmapToLocalFile(): "+e.toString());
		}
		return path;
	}
	
	public static String getCurrentDate(){
		Calendar c = Calendar.getInstance();
		SimpleDateFormat sdf = new SimpleDateFormat("dd-MM-yyyy HH:mm:ss");
		return sdf.format(c.getTime());
	}
	
	// AsyncTask genric
	public static void executeAsync(final AsyncListener listener){
		if(listener != null){
	    	new AsyncTask<Void, Void, Boolean>(){
	    		@Override
	    		protected Boolean doInBackground(Void... params) {
	    			boolean ok;
	    			try{
	    				ok = listener.runBackground();
	    			}
	    			catch(Exception e){
	    				ok = false;
	    				Log.e("Utils.executeAsync() /runBackground error: "+e.toString());
	    			}
	    			return ok;
	    		}
	    		
	    		@Override
	    		protected void onPostExecute(Boolean param){
	    			boolean ok = param.booleanValue();
	    			try{
	    				listener.runPost(ok);
	    			}
	    			catch(Exception e){
	    				Log.e("Utils.executeAsync() /runPost error: "+e.toString());
	    			}
	    		}
	    	}.execute();
		}
	}
	
	public interface AsyncListener {
	    public boolean runBackground();
	    public void runPost(boolean ok);
	}
}