<?php
defined('BASEPATH') OR exit('No direct script access allowed');

// This can be removed if you use __autoload() in config.php OR use Modular Extensions
require APPPATH . '/libraries/REST_Controller.php';

class Rest extends REST_Controller {

    static $rest_ok = 200; // Resposta correcta estàndard
    static $rest_ok_empty = 204; // Ha estat correcta, però encara no hi ha dades a la base de dades per ser retornades
    static $rest_error = 400; // Resposta incorrecta estàndard
    static $rest_error_unauthorized = 401; // Les dades per a l'autenticació no són correctes
    static $connection_ok = 200;
    static $result_deleted = "deleted";
    static $result_modified = "modified";

    public function __construct() {
        parent::__construct();

        $this->load->model('database');
    }

    function usuari_get() {
        $user = $this->checkUser();
        if ($user) {
            $this->codedResponse(self::$rest_ok, "OK", $user);
        }
    }

    function usuari_post() {
        $user = $this->checkUser();
        if ($user) {
            $data = $this->get_input_params();
            $rows = $this->database->updateUsuari($data);

            if ($rows > 0) {
                $this->codedResponse(self::$rest_ok, "Usuari actualitzat correctament");
            } else {
                $this->codedResponse(self::$rest_error, "Usuari no actualitzat correctament");
            }
        }
    }

    function empreses_get() {
        $user = $this->checkUser();
        if ($user) {
            $rows = $this->database->getEmpreses();

            if ($rows) {
                $this->codedResponse(self::$rest_ok, "OK", $rows);
            } else {
                $this->codedResponse(self::$rest_ok_empty, "Encara no hi ha empreses");
            }
        }
    }

    function empresesSync_get() {
        $user = $this->checkUser();
        if ($user) {
            $result = array(self::$result_deleted => array(), self::$result_modified => array());
            $this->codedResponse(self::$rest_ok, "OK", $result);
        }
    }

    function empresa_put() {
        $user = $this->checkUser();
        if ($user) {
            $id = $this->database->insertEmpresa($this->put());

            if ($id > 0) {
                $this->codedResponse(self::$rest_ok, "OK", array('id' => $id));
            } else {
                $this->codedResponse(self::$rest_error, "Empresa no insertada correctament");
            }
        }
    }

    function empresa_post() {
        $user = $this->checkUser();
        if ($user) {
            $rows = $this->database->updateEmpresa($this->post());

            if ($rows > 0) {
                $this->codedResponse(self::$rest_ok, "Empresa actualitzada correctament");
            } else {
                $this->codedResponse(self::$rest_error, "Empresa no actualitzada correctament");
            }
        }
    }

    function contactes_get() {
        $user = $this->checkUser();
        if ($user) {
            $id = $user["id"];
            $rows = $this->database->getContactes($id);

            if ($rows) {
                $this->codedResponse(self::$rest_ok, "OK", $rows);
            } else {
                $this->codedResponse(self::$rest_ok_empty, "Encara no hi ha contactes");
            }
        }
    }

    function contactesSync_get() {
        $user = $this->checkUser();
        if ($user) {
            $id = $user["id"];

            $result = array(self::$result_deleted => array(), self::$result_modified => array());
            $this->codedResponse(self::$rest_ok, "OK", $result);
        }
    }

    function contacte_put() {
        $user = $this->checkUser();
        if ($user) {
            $id = $this->database->insertContacte($this->put());

            if ($id > 0) {
                $this->codedResponse(self::$rest_ok, "OK", array('id' => $id));
            } else {
                $this->codedResponse(self::$rest_error, "Contacte no insertat correctament");
            }
        }
    }

    function contacte_post() {
        $user = $this->checkUser();
        if ($user) {
            $rows = $this->database->updateContacte($this->post());

            if ($rows > 0) {
                $this->codedResponse(self::$rest_ok, "Contacte actualitzat correctament");
            } else {
                $this->codedResponse(self::$rest_error, "Contacte no actualitzat correctament");
            }
        }
    }

    function contacte_delete() {
        $user = $this->checkUser();
        if ($user) {
            if (!$this->get('id')) {
                $this->codedResponse(self::$rest_error, "Falta l'id de contacte");
            } else {
                $rows = $this->database->deleteContacte($this->get('id'));

                if ($rows > 0) {
                    $this->codedResponse(self::$rest_ok, "Contacte eliminat correctament");
                } else {
                    $this->codedResponse(self::$rest_ok_empty, "Contacte no eliminat");
                }
            }
        }
    }

    function productes_get() {
        $user = $this->checkUser();
        if ($user) {
            $rows = $this->database->getProductes();

            if ($rows) {
                $this->codedResponse(self::$rest_ok, "OK", $rows);
            } else {
                $this->codedResponse(self::$rest_ok_empty, "Encara no hi ha productes");
            }
        }
    }

    function productesSync_get() {
        $user = $this->checkUser();
        if ($user) {
            $result = array(self::$result_deleted => array(), self::$result_modified => array());
            $this->codedResponse(self::$rest_ok, "OK", $result);
        }
    }

    function comandes_get() {
        $user = $this->checkUser();
        if ($user) {
            $id = $user["id"];
            $rows = $this->database->getComandes($id);

            if ($rows) {
                $this->codedResponse(self::$rest_ok, "OK", $rows);
            } else {
                $this->codedResponse(self::$rest_ok_empty, "Encara no hi ha comandes");
            }
        }
    }

    function comandesSync_get() {
        $user = $this->checkUser();
        if ($user) {
            $id = $user["id"];

            $result = array(self::$result_deleted => array(), self::$result_modified => array());
            $this->codedResponse(self::$rest_ok, "OK", $result);
        }
    }
    
    function comanda_put() {
        $user = $this->checkUser();
        if ($user) {
            $id = $this->database->insertComanda($this->put());

            if ($id > 0) {
                $this->codedResponse(self::$rest_ok, "OK", array('id' => $id));
            } else {
                $this->codedResponse(self::$rest_error, "Comanda no insertada correctament");
            }
        }
    }

    // -------------------------------------------

    protected function checkUser() {
        $auth = $this->basicAuth();
        $user = $this->database->getUsuari($auth['username'], $auth['password']);

        if ($user) {
            return $user;
        } else {
            $this->codedResponse(self::$rest_error_unauthorized, 'Unauthorized');
        }
    }

    protected function basicAuth() {
        $username = NULL;
        $password = NULL;

        // mod_php
        if ($this->input->server('PHP_AUTH_USER')) {
            $username = $this->input->server('PHP_AUTH_USER');
            $password = $this->input->server('PHP_AUTH_PW');
        }
        // most other servers
        elseif ($this->input->server('HTTP_AUTHENTICATION')) {
            if (strpos(strtolower($this->input->server('HTTP_AUTHENTICATION')), 'basic') === 0) {
                list($username, $password) = explode(':', base64_decode(substr($this->input->server('HTTP_AUTHORIZATION'), 6)));
            }
        }

        return array(
            "username" => $username,
            "password" => $password);
    }

    function codedResponse($code, $message, $result = null) {
        if (empty($result)) {
            $result = "";
        }
        $resp = array('status' => $code, 'message' => $message, 'result' => $result);

        $this->response($resp, self::$connection_ok);
    }

    function get_input_params() {
        $result = NULL;
        if (function_exists('json_decode')) {
            $content = str_replace('\"', '"', file_get_contents('php://input'));
            $content = utf8_encode($content);
            $result = json_decode($content, true);
        }
        return $result;
    }

}
